% This is the main MLE estimation file for estimating the parameters of the
% dynamic power-sharing model of Francois, Rainier, and Trebbi. This file
% estimates the parameters for a single country only.
%
% Chad Kendall, Apr 2012

% read in the excel data
% IMPORTANT: It is assumed that the data is sorted from the largest to
% smallest ethinicity within a country/time observation.

clear all;
[rawnumeric, rawtext, other] = xlsread('..\ethnicgroup_sorted.xls','A2:Y11750');

% lambda is the percentage of elites, assumed fixed across countries and
% time
lambda = 1;

% provide limits for start and end of country of interest in the data
% also number of ethnicities in country of interest (assumed fixed over time)

% Benin
%firstobs = 1;
%lastobs = 645;
%numeth = 15;

% Cameroon
% firstobs = 646;
% lastobs = 1548;
% numeth = 21;
% countryparam = [0.94 3.5E-13 251.7 0.0300]; % initial estimate from pooledestimation

% Congo
% firstobs = 1549;
% lastobs = 2808;
% numeth = 30;
% countryparam = [0.99 0.18/(1-0.18) 186.9 0.254];

% Cote d'Ivoire
%firstobs = 2809;
%lastobs = 3556;
%numeth = 17;

% Gabon
%firstobs = 3557;
%lastobs = 3996;
%numeth = 10;

% Ghana
%firstobs = 3997;
%lastobs = 4964;
%numeth = 22;

% Guinea
%firstobs = 4965;
%lastobs = 5360;
%numeth = 9;

% Kenya
%firstobs = 5361;
%lastobs = 6000;
%numeth = 16;

% Liberia
%firstobs = 6001;
%lastobs = 6660;
%numeth = 15;

% Nigeria
%firstobs = 6661;
%lastobs = 7391;
%numeth = 17;

% Republic of Congo
%firstobs = 7392;
%lastobs = 7821;
%numeth = 10;

% Sierra Leone
%firstobs = 7822;
%lastobs = 8423;
%numeth = 14;

% Tanzania
%firstobs = 8424;
%lastobs = 9829;
%numeth = 37;

% Togo
%firstobs = 9830;
%lastobs = 10709;
%numeth = 20;

% Uganda
firstobs = 10710;
lastobs = 11749;
numeth = 26;
countryparam = [1.682 100000 135.3 0.1004]; % initial estimate from pooledestimation

% extract the variables over the range of interest

popshare = rawnumeric(firstobs:lastobs,5)/100;
leader = rawnumeric(firstobs:lastobs,6);
leadertrans = rawnumeric(firstobs:lastobs,15);
govsize = rawnumeric(firstobs:lastobs,7);
govposition = rawnumeric(firstobs:lastobs,8);

% include all observations
includeobs = ones(size(popshare,1),1);

% calculate share of positions
govshare = govposition./govsize;

% normalize population
P=1;

% determine regimes (leader and length of leadership)
% first column is leader, second is length of leadership
% third column is first observation of regime change

idx = 1; % increments for each new regime

% loop over all data looking for leaders and regime changes
% NOTE: in the case of multiple leaders, only the first will be picked up

regimechange = 1; % start by assuming a change
startofregime = 1; % start of first regime
regimes(1,1) = NaN; % used to check that leader is found
regimes(1,3) = 1; % first transition associated with first observation
for i=(1:size(popshare,1))       
    % look for transition only every numeth rows
    if (regimechange == 0 && leadertrans(i) == 1 && mod(i,numeth) == 1)
        regimechange = 1;
        regimes(idx,2) = (i-startofregime)/numeth;
        startofregime = i;
        idx = idx + 1;
        regimes(idx,1) = NaN; % used to check that leader is found
        regimes(idx,3) = i; % observation at which transition occurred
    end;
    if regimechange == 1
        % looking for new leader
        if leader(i) == 1
            % found leader of new regime
            regimes(idx,1) = mod(i,numeth);
            if regimes(idx,1) == 0
                regimes(idx,1) = numeth;
            end;
            regimechange = 0;
        end;
    end;
    
end;

% length of last regime
% regimechange should be zero as we should have found the last start of
% regime and then the leader of that regime
if regimechange == 0 
    regimes(idx,2) = (size(popshare,1)+1-startofregime)/numeth;
else
    error('A leader was not found in the last regime.');
end;
    
% check that leaders were always found
if (sum(isnan(regimes(:,1))))
     error('A leader was not found for at least one regime.');
end;

% the parameter vector to be estimated is (first to last)

% Fprime - leadership payoff, transformed
% gamma_t - probability of successful coup (transformed)
% xi - parameter governing (beta) error distribution
% rprime = delta*epsilon*(1-r)/r/(1-delta) where delta is the
%    discount factor, epsilon is the probability of regime change, and r is
%    the factor by which patronage is destroyed when a revolution occurs


% fix alpha and epsilon at estimates from pooled estimation
alpha = 11.5222;
epsilon = 0.115/(1-0.115); % transformed from estimate obtained

myll = @(paramvec)splitlikelihood(paramvec,popshare,govshare,P,regimes,includeobs,numeth,lambda,0,0,[alpha epsilon],0);

% estimation is done using genetic algorithm as a first step and then
% fminsearch starting from the point found by ga

% ga optimization
% create bounds for initial population (not binding during optimization)
initrange = [0.25 0 40 0; 2 10 250 1];
% set options for ga algorithm
options = gaoptimset('PopulationSize',1000,'Generations',200,'TolFun',1E-10,'PopInitRange',initrange);

 % if an estimate from pooledestimation exists, place in initial population
    if (exist('countryparam','var')) ~= 0
        disp('Using estimate from pooledestimation in initial population.');
        options = gaoptimset(options,'InitialPopulation',countryparam);
    end;
    
% estimation
[paramestga fvalga exitflag] = ga(myll,4,options);

% local optimization (options and estimation)
fminopts = optimset('Display','off','TolX',1e-6,'TolFun',1e-10, 'MaxIter', 10000,'MaxFunEvals',10000);
[paramest,fval,fminflag] = fminsearch(myll,paramestga,fminopts);

if fminflag~=1
    disp('WARNING: fmin failed to converge.');
end;

% report likelihood and check that insider constraint is not violated
[ll,constflag] = splitlikelihood(paramest,popshare,govshare,P,regimes,includeobs,numeth,lambda,0,0,[alpha epsilon],1)

if constflag == 1
    error ('Insider constraint is violated at estimated parameters.');
end;

% calculate standard errors using the outer product of gradients
% likelihood function returns the vector of likelihoods rather than total
var = OPG('splitlikelihood',paramest',popshare,govshare,P,regimes,includeobs,numeth,lambda,1,0,[alpha epsilon],0);

seorig = sqrt(diag(inv(var))); % untransformed std errors

% create a matrix to apply the delta method to determine the standard
% errors of the transformed variables

% transform variables
gamma = abs(paramest(2))/(1+abs(paramest(2)));
F = paramest(1)*(1-gamma)/gamma;

J = zeros(size(paramest,2),size(paramest,2));
J(1,1) = (1-gamma)/gamma; % derivative of F wrt F'
J(1,2) = -paramest(1)/gamma^2*transformder(paramest(2)); % derivative of F wrt gamma
J(2,2) = transformder(paramest(2)); % derivative of transformation for gamma
J(3,3) = 1; % no transformation for xi
J(4,4) = 1; % no transformation for rprime

% transform and report estimates
disp('The estimated parameters are: F, gamma, xi, rprime, Fprime');

[F gamma paramest(3) paramest(4) paramest(1)]

disp('with standard errors:');
se = sqrt(diag(J*inv(var)*J')); %standard errors

[se(1) se(2) se(3) se(4) seorig(1)]