% This is the main MLE estimation file for estimating the parameters of the
% dynamic power-sharing model of Francois, Rainier, and Trebbi.
% Here, all countries are pooled together and we are estimating alpha and
% epsilon across countries as well as F, gamma, xi, and r individually for
% each country.
% 
%
% The optimization procedure proceeds by estimating alpha and epsilon only
% looking at regime transitions.
% It then estimates (using ga and then fmincon) F, gamma, xi, and rprime
% for each country. Then epsilon and rprime are used to recover r for each
% country. Finally, the standard errors are calculated assuming zero
% correlation between the regime parameters (alpha and epsilon) and the
% country parameters.

% Chad Kendall, July 2012

% read in the excel data
% IMPORTANT: It is assumed that the data is sorted from the largest to
% smallest ethinicity within a country/time observation.

clear all;

% set the following to skip the estimation and just use the estimates from
% file (for testing subsequent code)
skipest = 1;

% set the following to use the top government positions only
usetopshare = 0;

% set the following to look at various subsamples of the data
subsample = 0; % all observations
% subsample = 1; % democracies only (polity > 5)
% subsample = 2; % non-democracies only
% subsample = 3; % military dictatorships only
% subsample = 4; % non-military dictatorships only


% the country parameters are made global so that they can be accessed
% here and within the pooledlikelihood function and so that they maintain
% their values across function calls
% Note: care must be taken to ensure the parameters are the correct values
% for the type of estimation being done
% countryparam is made up of F, gamma, xi, rprime for each country (row)
global countryparam;

% read in data
[rawnumeric, rawtext, other] = xlsread('..\ethnicgroup_sorted.xls');

% index of last observation in the data
lastdataobs = 11749;

% create a 3-column array in which the first column is the number of
% ethnicities in the country and the second column is the observation at
% which the country starts in the data, the third is where the country ends

countries = [15 21 30 17 10 22 9 16 15 17 10 14 37 20 26; ...
    1 646 1549 2809 3557 3997 4965 5361 6001 6661 7392 7822 8424 9830 10710; ...
    645 1548 2808 3556 3996 4964 5360 6000 6660 7391 7821 8423 9829 10709 11749]';

% number of countries in the data
numcountries = size(countries,1);

% lambda is the percentage of elites, assumed fixed across countries and
% time
lambda = 1;

% extract the variables over the range of interest
popshare = rawnumeric(1:lastdataobs,5)/100;
leader = rawnumeric(1:lastdataobs,6);
leadertrans = rawnumeric(1:lastdataobs,15);
if usetopshare == 1
    govsize = rawnumeric(1:lastdataobs,10);
    govposition = rawnumeric(1:lastdataobs,11);
else
    govsize = rawnumeric(1:lastdataobs,7);
    govposition = rawnumeric(1:lastdataobs,8);
end;
year = rawnumeric(1:lastdataobs,2);
country = rawtext(2:lastdataobs+1,1);
group = rawtext(2:lastdataobs+1,4);

% read in polity data
[politynumeric, politytext, other] = xlsread('..\polityIV_2009.xls');
polityyear = politynumeric(:,1);
politycountry = politytext(2:size(politytext,1),2);
politypolity = politynumeric(:,4);

% read in military dictatorship data
[militarynumeric, militarytext, other] = xlsread('..\military.xls');
militaryyear = militarynumeric(:,1);
militarycountry = militarytext(2:size(militarytext(:,1),1));
militarymilitary = militarynumeric(:,2);

% create polity2 > 5 and military dictatorship dummies by matching on
% country and year in datasets
polity2 = zeros(lastdataobs,1);
military = zeros(lastdataobs,1);
polity2(:) = 2; % code as missing initially (will be excluded if remains as missing)
military(:) = 2; % code as missing initially (will be excluded if remains as missing)
% loop over countries and years to match up data
for c = 1:numcountries
    numperiods = (countries(c,3) - countries(c,2) + 1)/countries(c,1);
    for t=1:numperiods
        % index of observation
        obsidx = countries(c,2) + (t-1)*countries(c,1);
        % brute force search over all entries in polity/military spreadsheet, but
        % break when a match is found
        
        % polity search
        for polityidx=1:size(polityyear,1)
            if (isequal(country(obsidx),politycountry(polityidx)) && year(obsidx) == polityyear(polityidx))
                % found match
                % place polity dummy in country-year for all groups
                polity2(obsidx:obsidx+countries(c,1)-1)=politypolity(polityidx);
                break;
            end;
        end;
        
        % military search
        for militaryidx=1:size(militaryyear,1)
            if (isequal(country(obsidx),militarycountry(militaryidx)) && year(obsidx) == militaryyear(militaryidx))
                % found match
                % place military dummy in country-year for all groups
                military(obsidx:obsidx+countries(c,1)-1)=militarymilitary(militaryidx);
                break;
            end;
        end;
    end;
end;

% create a dummy variable which is set to 1 if the observation should be
% included based on the subsampling constants set at the beginning of the
% file

includeobs = zeros(lastdataobs,1);
if subsample == 1
    % only include democracies
    includeobs = (polity2 == 1);
elseif subsample == 2
    % only include non-democracies
    includeobs = (polity2 == 0);
elseif subsample == 3
    % only include military
    includeobs = (military == 1);
elseif subsample == 4
    % only include non-military
    includeobs = (military==0);
else
    % include all observations
    includeobs(:) = 1;
end;

% % uncomment the following to exclude Gabon, Guinea, and Sierra Leone
% includeobs(3557:3996) = 0;
% includeobs(4965:5360) = 0;
% includeobs(7822:8423) = 0;
    
% calculate share of positions
govshare = govposition./govsize;

% normalize population
P=1;

% determine regimes for each country
% first column is leader, second is length of leadership, third column
% is observation in which regime change occurred (relative to start of
% country)
% here, it is a 3d array in which the 3rd dimension is the country

for c=1:numcountries

    idx = 1; % increments for each new regime

    % loop over all data in the country looking for leaders and regime changes
    % NOTE: in the case of multiple leaders, only the first will be picked up
    
    % number of observations for the country
    numobs = countries(c,3) - countries(c,2) + 1;
    
    % first and last observations for the country
    firstobs = countries(c,2);
    lastobs = countries(c,3);
    % number of ethnicities for the country
    numeth = countries(c,1);
    
    regimechange = 1; % start by assuming a change
    startofregime = 1; % start of first regime
    regimes(1,1,c) = NaN; % used to check that leader is found
    regimes(1,3,c) = 1; % first transition associated with first observation
    for i=(1:numobs)
        % i is offest from start of country so need actual row of
        % observation
        row = firstobs + i - 1;
        
        % look for transition only every numeth rows
        if (regimechange == 0 && leadertrans(row) == 1 && mod(i,numeth) == 1)
            regimechange = 1;
            regimes(idx,2,c) = (i-startofregime)/numeth;
            startofregime = i;
            idx = idx + 1;
            regimes(idx,1,c) = NaN; % used to check that leader is found
            regimes(idx,3,c) = i; % observation at which transition occurred
        end;
        if regimechange == 1
            % looking for new leader
            if leader(row) == 1
                % found leader of new regime
                regimes(idx,1,c) = mod(i,numeth);
                if regimes(idx,1,c) == 0
                    regimes(idx,1,c) = numeth;
                end;
                regimechange = 0;
            end;
        end;

    end;

    % length of last regime
    % regimechange should be zero as we should have found the last start of
    % regime and then the leader of that regime
    if regimechange == 0 
        regimes(idx,2,c) = (numobs+1-startofregime)/numeth;
    else
        error('A leader was not found in the last regime.');
    end;

    % check that leaders were always found
    if (sum(isnan(regimes(:,1,c))))
         error('A leader was not found for at least one regime.');
    end;
end;

% the following estimates alpha and epsilon only

% initial values
alpha_0 = 0;
epsilon_0 = 0;
myll = @(paramvec)pooledlikelihood(paramvec,popshare,govshare,P,countries,regimes,includeobs,lambda,0,1,0,[]);
fminopts = optimset('Display','off','TolX',1e-6,'TolFun',1e-8, 'MaxIter', 1000,'MaxFunEvals',10000);
[aeest,ll,fminflag] = fminsearch(myll,[alpha_0 epsilon_0],fminopts);

% calculate the covariance matrix of alpha and epsilon for use in obtaining
% standard errors later

varae = OPG('pooledlikelihood',aeest',popshare,govshare,P,countries,regimes,includeobs,lambda,1,1,0,[]);

% now proceed with full estimation using the above estimates for alpha and
% epsilon

% within the pooled likelihood function, given the values of alpha and
% epsilon provided, the country parameters are estimated for each country:
%   Fprime - leadership payoff, transformed
%   gamma_t - probability of successful coup (transformed)
%   xi - parameter governing (beta) error distribution
%   rprime - the profit to cost ratio for a revolution (per elite member)

% perform a first estimation using previously determined estimates
% for alpha and epsilon and running ga for each country

% first load existing estimates if available to place into initial
% population, ensuring likelihood is at least as good as previous run

if usetopshare == 1
    paramfile = 'toppooledparam.mat';
else
    if subsample > 0
        paramfile = strcat('pooledparam',num2str(subsample),'.mat');
    else
        paramfile = 'pooledparam.mat';
    end;
end

fileexists = exist(paramfile);
if fileexists ~=0
    load(paramfile,'countryparam');
    disp('Loaded previous parameter values to use as initial population for ga.');
end;

if skipest == 0 % perform estimation
    % all of the actual estimation is done with the pooledlikelihood function
    llinit = pooledlikelihood([],popshare,govshare,P,countries,regimes,includeobs,lambda,0,0,1,[aeest(1) aeest(2)]);
    % report likelihood and country estimates
    disp('After ga, the likelihood and country estimates are:');
    llinit
    countryparam

    % now run fminsearch for each country starting from values obtained
    % with ga (using same alpha and epsilon again)
    llfinal = pooledlikelihood(aeest,popshare,govshare,P,countries,regimes,includeobs,lambda,0,0,2,[aeest(1) aeest(2)]);

    % save the parameters so that they can be placed into the initial
    % population in the next run
    save(paramfile,'countryparam');
end;

% copy all country parameters into a single row vector
for c=1:numcountries
    fullparam((c-1)*4+1:(c-1)*4+4) = countryparam(c,:);
end;

% report likelihood, check insider revolution constraints, and get model
% fit information and leadership premium
disp('After fminsearch, the likelihood is:');
[ll,llarray,constflag,sharedata,members] = pooledlikelihood(fullparam',popshare,govshare,P,countries,regimes,includeobs,lambda,0,0,0,[aeest(1) aeest(2)]);
ll
% if any insider constraint is violated, report an error
if sum(constflag) > 0
    constflag
    disp('Error: A one above indicates that the insider revolution constraint was violated for that country.');
end;

% calculate standard errors using the outer product of gradients
% likelihood function returns the vector of likelihoods rather than total
% this is the variance for the country parameters only: the covariance
% between alpha and epsilon is assumed zero
varcountry = OPG('pooledlikelihood',fullparam',popshare,govshare,P,countries,regimes,includeobs,lambda,1,0,0,[aeest(1) aeest(2)]);

% calculate r,gamma, and F for each country
epsilon = abs(aeest(2))/(1+abs(aeest(2)));
delta = 0.95; % calibrated

rarray=zeros(numcountries,1);
Farray=zeros(numcountries,1);
gammaarray=zeros(numcountries,1);
for c=1:numcountries
    rarray(c) = delta*epsilon/(fullparam(4*c)*(1-delta)+delta*epsilon);
    gammaarray(c) = abs(fullparam(4*(c-1)+2))/(1+abs(fullparam(4*(c-1)+2)));
    Farray(c) = fullparam(4*(c-1)+1)*(1-gammaarray(c))/gammaarray(c);
end;

% due to sub-sampling, rather than inverting the full var-cov matrix,
% we need to invert it country-by-country
% it has been verified that this results in the exact same standard errors
% as when inverting the whole matrix (with the full sample) which is
% because the inverse of a block diagonal matrix is equal to the matrix
% created by inverting each block individually

seorig = zeros(1,4*numcountries+2); % initialization
se = zeros(1,4*numcountries+2); % initialization
for c=1:numcountries
    
    % extract observations to include for the country
    countryinclude = includeobs(countries(c,2):countries(c,3));
    
    % calculate standard errors only if the country has some included
    % observations
    if sum(countryinclude) > 0
        
        % create a variance-covariance matrix for the country which includes
        % alpha and epsilon
        varfull = zeros(6,6);
        varfull(1:2,1:2) = varae;
        varfull(3:6,3:6) = varcountry(4*(c-1)+1:4*c,4*(c-1)+1:4*c);

        % now create a matrix for applying the delta method
        % each row corresponds to one parameter to be determined as a function of
        % the estimated parameters: F,r, epsilon, and gamma require transformations

        J = zeros(6,6);
        J(1,1) = 1; % no transformation for alpha
        J(2,2) = transformder(aeest(2)); % derivative of transformation for epsilon

        J(3,3) = (1-gammaarray(c))/gammaarray(c); % derivative of F wrt F'
        J(3,4) = -fullparam(4*(c-1)+1)/gammaarray(c)^2*transformder(fullparam(4*(c-1)+2)); % derivative of F wrt gamma
        J(4,4) = transformder(fullparam(4*(c-1)+2)); % derivative of transformation for gamma
        J(5,5) = 1; % no transformation for xi
        J(6,6) = -rarray(c)^2*(1-delta)/delta/epsilon; % derivative of r wrt rprime
        J(6,2) = rarray(c)*(1-rarray(c))*transformder(aeest(2))/epsilon;% derivative of r wrt to epsilon_t

        seorigtemp = sqrt(diag(inv(varfull))); % untransformed standard errors
        setemp = sqrt(diag(J*inv(varfull)*J')); % transformed standard errors
        % standard errors for alpha and epsilon are the same for each country
        seorig(1:2) = seorigtemp(1:2);
        se(1:2) = setemp(1:2);
        seorig(4*(c-1)+3:4*(c-1)+6) = seorigtemp(3:6);
        se(4*(c-1)+3:4*(c-1)+6) = setemp(3:6);
    end;
end;

% report estimates
disp('The estimated parameters (w/ std errors below) are: alpha, epsilon');
[aeest(1) epsilon]
se(1:2)
disp('F, gamma, xi, r, Fprime, rprime, negLL (per country):');
for c=1:numcountries
     % extract observations to include for the country
    countryinclude = includeobs(countries(c,2):countries(c,3));
    
    % report only if the country has some included observations
    if sum(countryinclude) > 0
        
        % report for a country
        disp('Reporting for country:');
        c
        [Farray(c) gammaarray(c) fullparam(4*(c-1)+3) rarray(c) fullparam(4*(c-1)+1)  fullparam(4*(c-1)+4) llarray(c)]
        [se(4*(c-1)+3) se(4*(c-1)+4) se(4*(c-1)+5) se(4*(c-1)+6) seorig(4*(c-1)+3) seorig(4*(c-1)+6)]
    end;
end

% this check is no longer necessary because the estimates seem to converge
% to the corner points on their own
% % ss a double-check that the estimates are in fact optimal, check the
% % extreme points of gamma (0 and 1), using the estimated value of all other
% % parameters: calculate the loglikelihood for these extreme points
% 
% g0param = fullparam; % copy of parameters for gamma = 0
% g1param = fullparam; % copy of parameters for gamma = 1a = 1
% 
% % replace gamma with 0 or 1
% % actually use values slightly away from 0 and 1 because model is not
% % strictly valid at these extremes
% for c=1:numcountries
%     g0param(4*(c-1)+2) = 1e-7/(1-1e-7); % transformed 1e-7
%     g1param(4*(c-1)+2) = (1-1e-7)/(1e-7); %transformed 1-1e-7
% end;
% 
% % calculate likelihoods
% [ll,llg0,constflag] = pooledlikelihood([aeest(1); aeest(2); g0param'],popshare,govshare,P,countries,regimes,lambda,0,0,0,[aeest(1) aeest(2)]);
% [ll,llg1,constflag] = pooledlikelihood([aeest(1); aeest(2); g1param'],popshare,govshare,P,countries,regimes,lambda,0,0,0,[aeest(1) aeest(2)]);
% 
% disp('The likelihoods for each country with gamma=0 are:');
% llg0'
% disp('The likelihoods for each country with gamma=1 are:');
% llg1'

% calculate model fit data and leadership premia

% initialization
leaderpremia = zeros(numcountries,1);
seleaderpremia = zeros(numcountries,1);
nu_hat_full = zeros(lastdataobs,1);
%pred_shares_full = zeros(lastdataobs,1);
% the following are used to get averages across all countries
totalnumperiods = 0;
totalldrpredshare = 0;
totalldrobsshare = 0;
totallrgpredshare = 0;
totallrgobsshare = 0;
totalwtpredcoalsize = 0;
totalwtobscoalsize = 0;
totalwtcorrectin = 0;
totalwtcorrectout = 0;

disp('Model fit data:');

% loop over all countries
for c=1:numcountries
    
    % extract observations to include for the country
    countryinclude = includeobs(countries(c,2):countries(c,3));
    
    % calculate model fit data only if the country has some included
    % observations
    if sum(countryinclude) > 0
        
        % number of observations
        numobs = countries(c,3) - countries(c,2) + 1;
        numincludedobs = sum(countryinclude);
            
        % copy predicted shares into a vector for all countries, used to
        % calculate goodness of fit outside the loop
        %pred_shares_full(countries(c,2):countries(c,3)) = sharedata(1:numobs,5,c);

        disp('Reporting for country:');
        c

        % calculate number of time periods (differs by country)
        numperiods = numincludedobs/countries(c,1);
        totalnumperiods = totalnumperiods + numperiods;
        numeth = countries(c,1); % number of ethnicities

        % calculate average leadership and largest shares (predicted vs. observed)
        avgldrpredshare = sum(sharedata(1:numobs,1,c).*countryinclude)/numperiods;
        avgldrobsshare = sum(sharedata(1:numobs,2,c).*countryinclude)/numperiods;
        avglrgpredshare = sum(sharedata(1:numobs,3,c).*countryinclude)/numperiods;
        avglrgobsshare = sum(sharedata(1:numobs,4,c).*countryinclude)/numperiods;
        totalldrpredshare = totalldrpredshare + avgldrpredshare*numperiods;
        totalldrobsshare = totalldrobsshare + avgldrobsshare*numperiods;
        totallrgpredshare = totallrgpredshare + avglrgpredshare*numperiods;
        totallrgobsshare = totallrgobsshare + avglrgobsshare*numperiods;
        disp ('Predicted and observed average leader shares:');
        [avgldrpredshare avgldrobsshare]
        disp ('Predicted and observed average largest group shares:');
        [avglrgpredshare avglrgobsshare]

        % calculate average coalition sizes weighted by pop'n
        avgwtpredcoalsize = sum(members(1:numobs,1,c).*members(1:numobs,3,c).*countryinclude)/numperiods;
        avgwtobscoalsize = sum(members(1:numobs,2,c).*members(1:numobs,3,c).*countryinclude)/numperiods;
        totalwtpredcoalsize = totalwtpredcoalsize + avgwtpredcoalsize*numperiods;
        totalwtobscoalsize = totalwtobscoalsize + avgwtobscoalsize*numperiods;
        disp ('Predicted and observed average coalition sizes (weighted by population):');
        [avgwtpredcoalsize avgwtobscoalsize]

        % calculate average correct predictions weighted by pop'n
        avgwtcorrectin = sum(members(1:numobs,3,c).*members(1:numobs,1,c).*members(1:numobs,2,c).*countryinclude)/numperiods;
        avgwtcorrectout = sum(members(1:numobs,3,c).*(1-members(1:numobs,1,c)).*(1-members(1:numobs,2,c)).*countryinclude)/numperiods;
        totalwtcorrectin = totalwtcorrectin + avgwtcorrectin*numperiods;
        totalwtcorrectout = totalwtcorrectout + avgwtcorrectout*numperiods;
        disp ('Average correct predictions (weighted by population): predict_in predict_out');
        [avgwtcorrectin avgwtcorrectout]
    
        % calculate leadership premia and their standard errors
        % members contains population share

        % vector of needed parameters: Fprime, gamma_t,rprime
        ldrparam = [fullparam(4*(c-1)+1) fullparam(4*(c-1)+2) fullparam(4*(c-1)+4)]';

        leaderpremia(c) = leaderpremium(ldrparam,members(1:countries(c,1),3,c),P,lambda);

        % to apply the delta method, we need the derivative of the
        % leaderpremium wrt Fprime,gamma_t, and rprime
        % we calculate these derivatvies numerically: the following returns the
        % derivative wrt each parameter as a column vector (i.e. the gradient)
        grad = numder('leaderpremium',ldrparam,members(1:countries(c,1),3,c),P,lambda);

        % calculate standard errors
        % need a vector where xi is skipped since it doesn't affect premium
        gradmod = [grad(1) grad(2) 0 grad(3)]';
        seleaderpremia(c) = sqrt(gradmod'*inv(varcountry(4*(c-1)+1:4*(c-1)+4,4*(c-1)+1:4*(c-1)+4))*gradmod);

    end;    
%     % plots of observed vs. predicted shares
%     
%     scatter(sharedata(1:numobs,5,c),sharedata(1:numobs,6,c));
%     xlabel('Predicted share');
%     ylabel('Observed share');
    
%     % LM test for correlation of nu across groups
%     
%     % get estimate of nu (obs - pred)
%     nu_hat = sharedata(1:numobs,6,c)-sharedata(1:numobs,5,c);
%     
%     % copy residuals into vector for all observations for export
%     nu_hat_full(countries(c,2):countries(c,3)) = nu_hat;
%     
%     % put into a panel format: each row is all ethnicities for one time
%     % period
%     clear nu_panel;
%     for t=1:numperiods
%         nu_panel(t,1:numeth) = nu_hat((t-1)*numeth+1:t*numeth)';
%     end;
%     
%     % demean each observation
%     meannu = sum(nu_panel)/numperiods;
%     clear nu_panel_demean;
%     for t=1:numperiods
%         nu_panel_demean(t,1:numeth) = nu_panel(t,1:numeth) - meannu;
%     end;
%     
%     % drop one group because of dependence (shares sum to one) and deal
%     % with just a numeth-1*numeth-1 matrix
%     
%     nu_panel_trunc = nu_panel_demean(:,1:numeth-1);
%     
%     % get estimate of std deviation for each group
%     se_nu = sqrt(diag(nu_panel_trunc'*nu_panel_trunc));
%     
%     % calculate correlation matrix of residuals
%     rho_nu = nu_panel_trunc'*nu_panel_trunc;
%     for k=1:numeth-1
%         for l=1:numeth-1
%             if rho_nu(k,l) ~= 0
%                 rho_nu(k,l) = rho_nu(k,l)/se_nu(k)/se_nu(l);
%             else
%                 rho_nu(k,l) = 0;
%             end;
%         end;
%     end;
%     
%     % calculate Breusch and Pagan LM statistic
%     LM_nu = 0; % initialization
%     for k=1:(numeth-2)
%         for l=(k+1):numeth-1
%             LM_nu = LM_nu + numperiods*rho_nu(k,l)^2-1;
%         end;
%     end;
%     disp('Breusch Pagan LM statistic for nu:');
%     LM_nu = LM_nu*sqrt(1/(numeth-1)/(numeth-2))
%     disp('with p-value:');
%     1-normcdf(LM_nu)
    
end;

disp('Pooled average predicted and observed leader shares:');
[totalldrpredshare totalldrobsshare]./totalnumperiods

disp('Pooled average predicted and observed largest shares:');
[totallrgpredshare totallrgobsshare]./totalnumperiods
    
disp('Pooled average weighted coalition size, predicted and observed:');
[totalwtpredcoalsize totalwtobscoalsize]./totalnumperiods

disp('Pooled average weighted correct predictions: in and out');
[totalwtcorrectin totalwtcorrectout]./totalnumperiods
    
disp('Leadership premia with standard errors below:');
leaderpremia'
seleaderpremia'

if (subsample==0)
        
    % LM test for correlation of leader transitions across countries

    % first need to create panel where each row corresponds to a time period
    % and each column to a country: each entry is 1 if a transition occurred
    % and 0 if not
    % this is complicated by missing data which is country-dependent
    % we use only 1965 and onwards because before 1965 several countries were
    % not yet independent, so leader transitions are not defined

    % start by extracting all the leader transitions in the data, including
    % pre-1965

    clear leadertrans_panel;
    for c=1:numcountries
        numobs = countries(c,3) - countries(c,2) + 1;
        numperiods = numobs/countries(c,1);
        numeth = countries(c,1); % number of ethnicities
        firstobs = countries(c,2);
        for t=1:numperiods
            leadertrans_panel(t,c)=leadertrans(firstobs+(t-1)*numeth);
        end;
    end;

    % now insert the missing leader transition data as provided by Francesco
    % via email July 15, 2012
    % we use the custom function insertelement which adds the data, shifting the
    % existing data down to higher time periods
    % 2 is used as a placeholder for undefined

    % Benin
    leadertrans_panel = insertelement(leadertrans_panel,1969-1960+1,1,0,1); % add row
    leadertrans_panel = insertelement(leadertrans_panel,1975-1960+1,1,0,0);
    % Cameroon
    leadertrans_panel = insertelement(leadertrans_panel,1969-1960+1,2,0,0);
    leadertrans_panel = insertelement(leadertrans_panel,1975-1960+1,2,0,0);
    % Congo
    leadertrans_panel = insertelement(leadertrans_panel,1960-1960+1,3,2,0);
    leadertrans_panel = insertelement(leadertrans_panel,1972-1960+1,3,0,0);
    leadertrans_panel = insertelement(leadertrans_panel,1974-1960+1,3,0,0);
    % Cote d'Ivoire
    leadertrans_panel = insertelement(leadertrans_panel,1975-1960+1,4,0,0);
    % Gabon
    leadertrans_panel = insertelement(leadertrans_panel,1975-1960+1,5,0,0);
    % Ghana
    leadertrans_panel = insertelement(leadertrans_panel,1975-1960+1,6,0,0);
    % Guinea
    leadertrans_panel = insertelement(leadertrans_panel,1975-1960+1,7,0,0);
    % Kenya
    leadertrans_panel = insertelement(leadertrans_panel,1960-1960+1,8,2,0);
    leadertrans_panel = insertelement(leadertrans_panel,1961-1960+1,8,2,0);
    leadertrans_panel = insertelement(leadertrans_panel,1962-1960+1,8,2,0);
    leadertrans_panel = insertelement(leadertrans_panel,1963-1960+1,8,0,0);
    leadertrans_panel = insertelement(leadertrans_panel,1975-1960+1,8,0,0);
    % Liberia
    leadertrans_panel = insertelement(leadertrans_panel,1975-1960+1,9,0,0);
    % Nigeria
    leadertrans_panel = insertelement(leadertrans_panel,1960-1960+1,10,2,0);
    leadertrans_panel = insertelement(leadertrans_panel,1975-1960+1,10,1,0);
    % Republic of Congo
    leadertrans_panel = insertelement(leadertrans_panel,1969-1960+1,11,1,0);
    leadertrans_panel = insertelement(leadertrans_panel,1975-1960+1,11,0,0);
    % Sierra Leone
    leadertrans_panel = insertelement(leadertrans_panel,1972-1960+1,12,0,0);
    leadertrans_panel = insertelement(leadertrans_panel,1975-1960+1,12,0,0);
    % Tanzania
    leadertrans_panel = insertelement(leadertrans_panel,1960-1960+1,13,2,0);
    leadertrans_panel = insertelement(leadertrans_panel,1961-1960+1,13,2,0);
    leadertrans_panel = insertelement(leadertrans_panel,1962-1960+1,13,2,0);
    leadertrans_panel = insertelement(leadertrans_panel,1963-1960+1,13,2,0);
    leadertrans_panel = insertelement(leadertrans_panel,1964-1960+1,13,2,0);
    leadertrans_panel = insertelement(leadertrans_panel,1972-1960+1,13,0,0);
    leadertrans_panel = insertelement(leadertrans_panel,1974-1960+1,13,0,0);
    % Togo
    leadertrans_panel = insertelement(leadertrans_panel,1975-1960+1,14,0,0);
    % Uganda
    leadertrans_panel = insertelement(leadertrans_panel,1960-1960+1,15,2,0);
    leadertrans_panel = insertelement(leadertrans_panel,1961-1960+1,15,2,0);
    leadertrans_panel = insertelement(leadertrans_panel,1962-1960+1,15,2,0);
    leadertrans_panel = insertelement(leadertrans_panel,1972-1960+1,15,0,0);
    leadertrans_panel = insertelement(leadertrans_panel,1974-1960+1,15,0,0);

    % drop 1960-1964 because of pre-independence for some countries
    leadertrans_panel_trunc = leadertrans_panel(6:size(leadertrans_panel,1),:);

    clear trans_panel_demean;
    % demean each observation
    meantrans = sum(leadertrans_panel_trunc)/size(leadertrans_panel_trunc,1);
    for t=1:size(leadertrans_panel_trunc,1)
        trans_panel_demean(t,1:numcountries) = leadertrans_panel_trunc(t,1:numcountries) - meantrans;
    end;

    % get estimate of std deviation for each country
    se_trans = sqrt(diag(trans_panel_demean'*trans_panel_demean));

    % calculate correlation matrix of transitions
    rho_trans = trans_panel_demean'*trans_panel_demean;
    for k=1:numcountries
        for l=1:numcountries
            if rho_trans(k,l) ~= 0
                rho_trans(k,l) = rho_trans(k,l)/se_trans(k)/se_trans(l);
            else
                rho_trans(k,l) = 0;
            end;
        end;
    end;

    % calculate Breusch and Pagan LM statistic
    LM_trans = 0; % initialization
    for k=1:(numcountries-1)
        for l=(k+1):numcountries
            LM_trans = LM_trans + size(leadertrans_panel_trunc,1)*rho_trans(k,l)^2-1;
        end;
    end;
    disp('Breusch Pagan LM statistic for epsilon:');
    LM_trans = LM_trans*sqrt(1/numcountries/(numcountries-1))
    disp('with p-value:');
    1-normcdf(LM_trans)
end;

% export residuals for running serial correalation test in Stata
% if (subsample == 0 && usetopshare == 0)
%     xlswrite('residuals.xls',country);
%     xlswrite('residuals.xls',year,1,'B1');
%     xlswrite('residuals.xls',group,1,'C1');
%     xlswrite('residuals.xls',nu_hat_full,1,'D1');
% end;

% calculate goodness of fit: Pearson's chi-squared test

% % set bin sizes
% % the first tiny bin is used just to capture shares of zero
% binedges = [0 0.00001 0.01 0.015 0.02 0.025 0.03 0.05 0.07 0.1 0.15 1];
% %predcounts = histc(pred_shares_full(countries(c,2):countries(c,3)),binedges);
% %obscounts = histc(govshare(countries(c,2):countries(c,3)),binedges);
% predcounts = histc(pred_shares_full,binedges);
% obscounts = histc(govshare,binedges);
% nobs = lastdataobs;
% %nobs = countries(c,3)-countries(c,2) + 1;
% numbins = size(binedges,2) - 1;
% % need to drop the last count element which is the number which match the
% % last bin exactly
% % also transform to probabilities by dividing by number of observations
% disp('The binned frequencies of predicted and observed shares are:');
% predfreq = predcounts(1:numbins)./nobs;
% obsfreq = obscounts(1:numbins)./nobs;
% [predfreq obsfreq]
% Pstatistic = nobs*sum((predfreq-obsfreq).^2./predfreq);
% disp('Pearsons statistic is:');
% Pstatistic
% disp('with p-value:');
% 1-chi2cdf(Pstatistic,(numbins-1))
        
% compare model fits to simple models

% number of included observations
disp('Number of observations in the sample:');
nobs = sum(includeobs)

% full model
% for each country we need the log-likelihoods for each observation
llobs_full = -pooledlikelihood(fullparam',popshare,govshare,P,countries,regimes,includeobs,lambda,1,0,0,[aeest(1) aeest(2)]);
disp ('Sum of likelihoods for full model:');

% remove non-included observations
llobs_full(includeobs==0) = [];
ll_full = ones(1,nobs)*llobs_full

% model in which coalitions are determined randomly
randxi=zeros(numcountries,1); % initialization
llobs_rand=zeros(lastdataobs,1); % initialization
for c=1:numcountries
    
    % country specific constants
    numeth = countries(c,1); % number of ethnicities
    firstobs = countries(c,2);
    lastobs = countries(c,3);
    
    % estimate each country's likelihood using random model and fminsearch
    randxi_0 = 25; % starting point for optimization
    myll = @(xi)simplelikelihood(xi,popshare(firstobs:lastobs),...
                govshare(firstobs:lastobs),regimes(:,:,c),includeobs(firstobs:lastobs),numeth,0,0);
    [randxi(c),randll,randfminflag] = fminsearch(myll,randxi_0,fminopts);
    if randfminflag ~=1
        disp('fmincon failed to converge in random model for country:');
        c
    end;
    
    % get individual log-likelihoods
    llobs_rand(firstobs:lastobs) = -simplelikelihood(randxi(c),popshare(firstobs:lastobs),...
                govshare(firstobs:lastobs),regimes(:,:,c),includeobs(firstobs:lastobs),numeth,0,1);
end

% remove non-included observations
llobs_rand(includeobs==0) = [];

disp('Sum of likelihoods from random model:');
ll_rand = ones(1,nobs)*llobs_rand

% calculate standard deviation of likelihood ratios of individual
% observations
var_lr_rand = (llobs_full-llobs_rand)'*(llobs_full-llobs_rand)/nobs - ...
    (sum(llobs_full-llobs_rand)/nobs)^2;

disp('Vuong test statistic and p-value for random model:');
% we are adjusting for degrees of freedom in each model
randvuong = ll_full-ll_rand - ((60/2)*log(nobs)-(15/2)*log(nobs));
randvuong = randvuong/sqrt(nobs)/sqrt(var_lr_rand)
if randvuong > 0
    2*(1-normcdf(randvuong))
else
    2*normcdf(randvuong)
end;

% calculate Clarke statistic
% it compares the number of positive differences in individual likelihoods
% to 50%, which would indicate equal goodness of fit
% we correct for degrees of freedom again
llobs_full_corr = llobs_full - 60/2/nobs*log(nobs);
llobs_rand_corr = llobs_rand - 15/2/nobs*log(nobs);

disp('Clarke test statistic and p-value for random model:');
randclarke = sum((llobs_full_corr-llobs_rand_corr)>0)
if randclarke > nobs/2
    2*(1-binocdf(randclarke,nobs,0.5))
else
    2*binocdf(randclarke,nobs,0.5)
end;

% model in which leader gets entire share

ldronlyxi=zeros(numcountries,1); % initialization
llobs_ldronly=zeros(lastdataobs,1); % initialization
for c=1:numcountries
    
    % country specific constants
    numeth = countries(c,1); % number of ethnicities
    firstobs = countries(c,2);
    lastobs = countries(c,3);
    
    % estimate each country's likelihood using leader only model and fminsearch
    ldronlyxi_0 = 25; % starting point for optimization
    myll = @(xi)simplelikelihood(xi,popshare(firstobs:lastobs),...
                govshare(firstobs:lastobs),regimes(:,:,c),includeobs(firstobs:lastobs),numeth,1,0);
    [ldronlyxi(c),ldronlyll,ldronlyfminflag] = fminsearch(myll,ldronlyxi_0,fminopts);
    if ldronlyfminflag ~=1
        disp('fmincon failed to converge in leader only model for country:');
        c
    end;
    
    % get individual log-likelihoods
    llobs_ldronly(firstobs:lastobs) = -simplelikelihood(ldronlyxi(c),popshare(firstobs:lastobs),...
                govshare(firstobs:lastobs),regimes(:,:,c),includeobs(firstobs:lastobs),numeth,1,1);
end

% remove non-included observations
llobs_ldronly(includeobs==0) = [];

disp('Sum of likelihoods from model in which leader takes all:');
ll_ldronly = ones(1,nobs)*llobs_ldronly

% calculate standard deviation of likelihood ratios of individual
% observations
var_lr_ldronly = (llobs_full-llobs_ldronly)'*(llobs_full-llobs_ldronly)/nobs - ...
    (sum(llobs_full-llobs_ldronly)/nobs)^2;

disp('Vuong test statistic and p-value for leader only model:');
% we are adjusting for degrees of freedom in each model
ldrvuong = ll_full-ll_ldronly - ((60/2)*log(nobs)-(15/2)*log(nobs));
ldrvuong = ldrvuong/sqrt(nobs)/sqrt(var_lr_ldronly)
if ldrvuong > 0
    2*(1-normcdf(ldrvuong))
else
    2*normcdf(ldrvuong)
end;
        
% calculate Clarke statistic
% it compares the number of positive differences in individual likelihoods
% to 50%, which would indicate equal goodness of fit
% we correct for degrees of freedom again
llobs_ldronly_corr = llobs_ldronly - 15/2/nobs*log(nobs);

disp('Clarke test statistic and p-value for leader only model:');
ldrclarke = sum((llobs_full_corr-llobs_ldronly_corr)>0)
if ldrclarke > nobs/2
    2*(1-binocdf(ldrclarke,nobs,0.5))
else
    2*binocdf(ldrclarke,nobs,0.5)
end;

% window dressing model

windowparam=zeros(numcountries,2); % initialization
llobs_window=zeros(lastdataobs,1); % initialization
for c=1:numcountries
    
    % country specific constants
    numeth = countries(c,1); % number of ethnicities
    firstobs = countries(c,2);
    lastobs = countries(c,3);
    
    % estimate each country's likelihood using model and fminsearch
    windowparam_0 = [0.5 25]; % starting point for optimization
    myll = @(param)windowlikelihood(param,popshare(firstobs:lastobs),...
                govshare(firstobs:lastobs),regimes(:,:,c),includeobs(firstobs:lastobs),numeth,0);
    [windowparam(c,:),windowll,windowfminflag] = fminsearch(myll,windowparam_0,fminopts);
    if windowfminflag ~=1
        disp('fmincon failed to converge in window dressing model for country:');
        c
    end;
    
    % get individual log-likelihoods
    llobs_window(firstobs:lastobs) = -windowlikelihood(windowparam(c,:),popshare(firstobs:lastobs),...
                govshare(firstobs:lastobs),regimes(:,:,c),includeobs(firstobs:lastobs),numeth,1);
end

disp('Estimates for window dressing model (weight, xi):');
windowparam

% remove non-included observations
llobs_window(includeobs==0) = [];

disp('Sum of likelihoods from window dressing model:');
ll_window = ones(1,nobs)*llobs_window

% calculate standard deviation of likelihood ratios of individual
% observations
var_lr_window = (llobs_full-llobs_window)'*(llobs_full-llobs_window)/nobs - ...
    (sum(llobs_full-llobs_window)/nobs)^2;

disp('Vuong test statistic and p-value for window dressing model:');
% we are adjusting for degrees of freedom in each model
windowvuong = ll_full-ll_window - ((60/2)*log(nobs)-(30/2)*log(nobs));
windowvuong  = windowvuong /sqrt(nobs)/sqrt(var_lr_window)
if windowvuong > 0
    2*(1-normcdf(windowvuong))
else
    2*normcdf(windowvuong)
end;
        
% calculate Clarke statistic
% it compares the number of positive differences in individual likelihoods
% to 50%, which would indicate equal goodness of fit
% we correct for degrees of freedom again
llobs_window_corr = llobs_window - 30/2/nobs*log(nobs);

disp('Clarke test statistic and p-value for window dressing model:');
windowclarke = sum((llobs_full_corr-llobs_window_corr)>0)
if windowclarke > nobs/2
    2*(1-binocdf(windowclarke,nobs,0.5))
else
    2*binocdf(windowclarke,nobs,0.5)
end;

% polarization model in which shares are based on relative sizes

polsharexi=zeros(numcountries,1); % initialization
llobs_polshare=zeros(lastdataobs,1); % initialization
for c=1:numcountries
    
    % country specific constants
    numeth = countries(c,1); % number of ethnicities
    firstobs = countries(c,2);
    lastobs = countries(c,3);
    
    % estimate each country's likelihood using model and fminsearch
    polsharexi_0 = 25; % starting point for optimization
    myll = @(xi)simplelikelihood(xi,popshare(firstobs:lastobs),...
                govshare(firstobs:lastobs),regimes(:,:,c),includeobs(firstobs:lastobs),numeth,2,0);
    [polsharexi(c),polsharell,polsharefminflag] = fminsearch(myll,polsharexi_0,fminopts);
    if polsharefminflag ~=1
        disp('fmincon failed to converge in polarization share model for country:');
        c
    end;
    
    % get individual log-likelihoods
    llobs_polshare(firstobs:lastobs) = -simplelikelihood(polsharexi(c),popshare(firstobs:lastobs),...
                govshare(firstobs:lastobs),regimes(:,:,c),includeobs(firstobs:lastobs),numeth,2,1);
end

% remove non-included observations
llobs_polshare(includeobs==0) = [];

disp('Sum of likelihoods from polarization share model:');
ll_polshare = ones(1,nobs)*llobs_polshare

% calculate standard deviation of likelihood ratios of individual
% observations
var_lr_polshare = (llobs_full-llobs_polshare)'*(llobs_full-llobs_polshare)/nobs - ...
    (sum(llobs_full-llobs_polshare)/nobs)^2;

disp('Vuong test statistic and p-value for polarization share model:');
% we are adjusting for degrees of freedom in each model
polsharevuong = ll_full-ll_polshare - ((60/2)*log(nobs)-(15/2)*log(nobs));
polsharevuong  = polsharevuong /sqrt(nobs)/sqrt(var_lr_polshare)
if polsharevuong > 0
    2*(1-normcdf(polsharevuong))
else
    2*normcdf(polsharevuong)
end;
        
% calculate Clarke statistic
% it compares the number of positive differences in individual likelihoods
% to 50%, which would indicate equal goodness of fit
% we correct for degrees of freedom again
llobs_polshare_corr = llobs_polshare - 15/2/nobs*log(nobs);

disp('Clarke test statistic and p-value for polarization share model:');
polshareclarke = sum((llobs_full_corr-llobs_polshare_corr)>0)
if polshareclarke > nobs/2
    2*(1-binocdf(polshareclarke,nobs,0.5))
else
    2*binocdf(polshareclarke,nobs,0.5)
end;

% polarization model using country-level polarization
% in this model, we have cross-country restrictions forcing all countries
% to have the same estimated model parameter, polar (xi is per country)

% calculate polarization levels for each country
polarization=zeros(numcountries,1); % initializatino

for c=1:numcountries

    numeth = countries(c,1); % number of ethnicities

    % calculate polarization for country assuming K=1, alpha = 4/5
    polarization(c) = 0;
    for i = 1:numeth
        for j = 1:numeth
            if i~=j
                polarization(c) = polarization(c) + popshare(i)^(1+4/5)*popshare(j);
            end;
        end;
    end;

end

% estimation is done across all countries at once
% parameter vector is made up of a model parameter common to all
% countries and each country's error parameter (xi)

llobs_polar=zeros(lastdataobs,1); % initialization

% estimate the likelihood for all countries at once using the model and
% fminsearch
fminopts = optimset('Display','iter','TolX',1e-6,'TolFun',1e-7, 'MaxIter', 2000,'MaxFunEvals',10000);

% starting points have been optimized from previous iterations
if usetopshare == 1
    polarparam_0 = [-0.0242   12.0969   20.2273   21.5806    9.3770   13.5049    8.4518   12.4075   16.3696    2.8977   20.6887   10.7199   12.5424   25.2727   10.8826   17.5035]; % starting point for optimization
else
    if subsample == 4
        polarparam_0 = [-0.0052   49.6146  192.3832  136.9738  117.4611   57.0355   82.8609   48.5890  168.5602    3.3261   74.8660   55.9074   51.0451  113.8374   16.9516  182.1038];
    else
        polarparam_0 = [-0.0102   50.4226  196.9414  165.9181  116.8664   57.1601   73.2158   45.5411  170.8572    6.8278  110.6114   37.3508   51.5981  102.5734   52.2555  123.4385]; % starting point for optimization
    end;
end;
myll = @(param)polarizationwrap(param,numcountries,polarization,popshare,...
            govshare,countries,includeobs,0);
[polarparam,polarll,polarfminflag] = fminsearch(myll,polarparam_0,fminopts);
if polarfminflag ~=1
    disp('fmincon failed to converge in country polarization model');
end;

% get individual log-likelihoods
llobs_polar = -polarizationwrap(polarparam,numcountries,polarization,popshare,...
            govshare,countries,includeobs,1);

disp('Estimates for country polarization model (polarization coef. and xi per country):');
polarparam

% remove non-included observations
llobs_polar(includeobs==0) = [];

disp('Sum of likelihoods from country polarization model:');
ll_polar = ones(1,nobs)*llobs_polar

% calculate standard deviation of likelihood ratios of individual
% observations
var_lr_polar = (llobs_full-llobs_polar)'*(llobs_full-llobs_polar)/nobs - ...
    (sum(llobs_full-llobs_polar)/nobs)^2;

disp('Vuong test statistic and p-value for polarization share model:');
% we are adjusting for degrees of freedom in each model
polarvuong = ll_full-ll_polar - ((60/2)*log(nobs)-(16/2)*log(nobs));
polarvuong  = polarvuong /sqrt(nobs)/sqrt(var_lr_polar)
if polarvuong > 0
    2*(1-normcdf(polarvuong))
else
    2*normcdf(polarvuong)
end;

% calculate Clarke statistic
% it compares the number of positive differences in individual likelihoods
% to 50%, which would indicate equal goodness of fit
% we correct for degrees of freedom again
llobs_polar_corr = llobs_polar - 16/2/nobs*log(nobs);

disp('Clarke test statistic and p-value for country polarization model:');
polarclarke = sum((llobs_full_corr-llobs_polar_corr)>0)
if polarclarke > nobs/2
    2*(1-binocdf(polarclarke,nobs,0.5))
else
    2*binocdf(polarclarke,nobs,0.5)
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% additional counterfactuals for Cameroon/Nigeria
% and for Ghana/Togo (R&R ECMA)
% there are 3 different counterfactuals C/N corresponding to
% the different ways the referendum could have turned out
% there is 1 counterfactual for G/T

for i=1:4 % four counterfactuals
        
        % read in the counterfactual pop'n shares
        if i==1
            [cfnumeric, cftext, other] = xlsread('..\CameroonNigeria.xls','Opposite','A2:G1635');
            disp('Results for Cameroon/Nigeria opposite');
        elseif i==2
            [cfnumeric, cftext, other] = xlsread('..\CameroonNigeria.xls','All Nigeria','A2:G1635');
            disp('Results for all join Nigeria');
        elseif i==3
            [cfnumeric, cftext, other] = xlsread('..\CameroonNigeria.xls','All Cameroon','A2:G1635');  
            disp('Results for all join Cameroon');
        else
            [cfnumeric, cftext, other] = xlsread('..\TogoGhana.xls','Counter','A2:G1849'); 
            disp('Results for Ghana/Togo counterfactual');
        end;
            
        % we'll calculate for all countries even though only interested in
        % CAM/NIG or TOG/GHA

        % overwrite pop'n shares for CAM and NIG
        if i<=3
            popshare(countries(2,2):countries(2,3))=cfnumeric(1:903,5)/100;
            popshare(countries(10,2):countries(10,3))=cfnumeric(904:1634,5)/100;
        else % overwrite Togo and Ghana
            popshare(countries(6,2):countries(6,3))=cfnumeric(1:968,5)/100;
            popshare(countries(14,2):countries(14,3))=cfnumeric(969:1848,5)/100;
        end;
        % calculate counterfactual share data
        [cfll,cfllarray,cfconstflag,cfsharedata,cfmembers] = pooledlikelihood(fullparam',popshare,govshare,P,countries,regimes,includeobs,lambda,0,0,0,[aeest(1) aeest(2)]);

        % if any insider constraint is violated, report an error
        if sum(cfconstflag) > 0
            cfconstflag
            disp('Error: A one above indicates that the insider revolution constraint was violated for that country.');
        end;

    % countries to loop over
    if i<=3
        loopc = [2,10];
    else
        loopc = [6,14];
    end;
    
    for c=loopc
        if c==2
            disp('Counterfactuals for CAM');
        elseif c == 10
            disp('Counterfactuals for NIG');
        elseif c == 6
            disp('Counterfactuals for GHA');
        else
            disp('Counterfactuals for TOG');
        end;
            
        numobs = countries(c,3) - countries(c,2) + 1;
        numperiods = numobs/countries(c,1);

        % calculate counterfactual average leadership and largest shares
        avgldrpredshare = sum(cfsharedata(1:numobs,1,c))/numperiods;
        avglrgpredshare = sum(cfsharedata(1:numobs,3,c))/numperiods;

        disp ('Predicted average leader shares:');
        avgldrpredshare
        disp ('Predicted average largest group shares:');
        avglrgpredshare

        % calculate average coalition sizes weighted by pop'n
        avgwtpredcoalsize = sum(cfmembers(1:numobs,1,c).*cfmembers(1:numobs,3,c))/numperiods;
        disp ('Predicted average coalition sizes (weighted by population):');
        avgwtpredcoalsize
    end;
end;



