function [ll,llarray,constflag,sharedata,members]= pooledlikelihood(paramvec,popshare,govshare,P,countries,regimes,includeobs,lambda,vectorize,estregimes,estoption,ae)

% This function estimates the parameters of the dynamic coalition model,
% pooling the data across all countries. It calls likelihood for each
% country and then sums the likelihood contributions of all countries.
% A hierarchical optimization approach is used so that alpha and epsilon
% are estimated globally across countries, but F, gamma, epsilon, and rprime are
% estimated within this function for each country separately (given the
% current values of alpha and epsilon).
% 
% constflag is a vector (one bit per country) which is set to one if the
% insider revolution constraint is violated for the country, 0 otherwise.
%
% sharedata is a 3d array: first two dimensions are predicted and
% observed shares (see splitlikelihood.m), third dimension is country
%
% members is a 3d array: first two dimensions are predicted and observed
% coalition membership as well as population share, third dimension is country
%
%
% Inputs:
%
% estregimes: set to 1 to estimate the regime-related parameters only and
% set to 0 to estimate country parameters; when set to 1, only the likelihood
% from the regime transitions is calculated and returned (country
% estimations are not done)
%
% paramvec: the vector of parameters to be estimated
%
% estregimes == 1
%
%       alpha - parameter in probability dist'n over leadership
%       epsilon_t - probability of transition event (transformed)
%
% estregimes == 0
%
%       for each country:
%           Fprime - leadership payoff, transformed
%           gamma_t - probability of successful coup (transformed)
%           xi - parameter governing (beta) error distribution
%           rprime = delta*epsilon*(1-r)/r/(1-delta) where delta is the
%               discount factor, epsilon is the probability of regime change, and r is
%               the factor by which patronage is destroyed when a revolution occurs
%
%
%
% popshare: vector of the population share for each ethnicity-time obs
%
% govshare: observed share of government positions for each ethnicity-time
% obs
%
% P : total country population (fixed across time)
%
% leader: whether or not an ethnicity is the leader in the ethnicity-time
% obs
%
% countries: a 3 column matrix where first column lists the number of
% ethnicities in each country, the second column lists the first
% observation for each country, the third column lists the last;
%
% regimes: three column vector where first column is leader, second
% column is length of leadership, third column is observation in which
% transition occurred; one row per regime
% here, it is a 3d array in which the 3rd dimension is the country
%
% includeobs : a '1' for each observation that should be included in
% calculatng the likelihood
%
% lambda : percentage of elites in the population (same for all
% ethnicities)
%
% vectorize : set to 1 to return the vector of likelihoods rather than the
% summed likelihood (used for computing standard errors);
%
% estoption : estimation options (valid only when estregimes==0)
%   0 : no country estimation is done, the likelihood is simply calculated
%   given alpha, epsilon, and the current country parameters;
%   used to get standard errors, check insider revolution constraints, and
%   get model fit data
%   1 : genetic algorithm is run for each country (alpha and epsilon are
%   fixed to values in ae)
%   2 : fminsearch is run for each country, starting from previous
%   estimate (alpha and epsilon fixed to values in ae)
%
% ae: 2 entry vector: [alpha epsilon]; used with estregimes == 0

% the country parameters are made global so that they can be accessed
% outside of this function and so that they hold their values between
% function calls
% Note: care must be taken to ensure the parameters are the correct values
% for the type of estimation being done
% countryparam is made up of F, gamma, xi, rprime for each country (row)
global countryparam;

% lower and upper bounds for ga initial population
initrange = [0.25 0 40 0; 2 10 250 1];

% set options for ga algorithm
gaoptions = gaoptimset('PopulationSize', 1000,'Generations',200,'TolFun',1E-10,'PopInitRange',initrange);

% set options for fminsearch
fminopts = optimset('Display','off','TolX',1e-6,'TolFun',1e-10, 'MaxIter', 10000,'MaxFunEvals',10000);

% last observation in the data
lastdataobs = size(popshare,1);

% number of countries in the data
numcountries = size(countries,1);

% loop over each country to calculate the likelihood, placing into an array
% either the likelihood of each observation or for each country, depending
% on vectorize
% if estoption is 1 or 2, an estimation of each country's parameters is
% done to get the optimal likelihood

% initialization
if vectorize == 1
    if estregimes == 1
        % don't know the size in advance here since it is equal to the number
        % of regimes
        llarray = [];
    else
        llarray = zeros(lastdataobs,1);
    end;
else
    llarray = zeros(numcountries,1);
end;

% vector of insider constraint bits (per country)
constflag = zeros(numcountries,1);

% initializatin of model fit arrays
sharedata = [];
members = [];

for c=1:numcountries
    
    % place the old estimate into the initial population if it exists
    if size(countryparam,1) >= c
        gaoptions = gaoptimset(gaoptions,'InitialPopulation',countryparam(c,:));
    end;
    
    % number of observations for the country
    numobs = countries(c,3) - countries(c,2) + 1;
    
    % first and last observations for the country
    firstobs = countries(c,2);
    lastobs = countries(c,3);
    % number of ethnicities for the country
    numeth = countries(c,1);
    
    % only estimate if there are observations to use (except when
    % estimating alpha and epsilon in which case all countries are used)
    if (sum(includeobs(firstobs:lastobs)) > 0 || estregimes == 1)
        
        % here we have to remove potential rows of zeros in the regimes for the
        % country
        countryregimestemp = regimes(:,:,c);
        nonzero = find(countryregimestemp(:,1)); % indices of all non-zero rows
        nonzeromax = size(nonzero,1);
        countryregimes = countryregimestemp(1:nonzeromax,:);

        % calculate likelihood for the country
        if vectorize == 1
            % here, no estimation is done as this is for obtaining standard
            % errors only
            if estregimes == 1
                % find current size of array (number of regimes so far)
                currentsize = size(llarray,1);
                llarray((currentsize+1):(currentsize+size(countryregimes,1)),1) = ...
                    splitlikelihood(paramvec,popshare(firstobs:lastobs),govshare(firstobs:lastobs),...
                    P,countryregimes,includeobs(firstobs:lastobs),numeth,lambda,1,1,[0 0],0);
            else           
                % use parameters for specific country, alpha and epsilon
                % come from ae
                llarray(firstobs:lastobs) = splitlikelihood([paramvec((c-1)*4+1:(c-1)*4+4)'],...
                    popshare(firstobs:lastobs),govshare(firstobs:lastobs),...
                    P,countryregimes,includeobs(firstobs:lastobs),numeth,lambda,1,0,[ae(1) ae(2)],0);
            end;
        else
            if estregimes == 1
                llarray(c) = splitlikelihood(paramvec,popshare(firstobs:lastobs),...
                govshare(firstobs:lastobs),P,countryregimes,includeobs(firstobs:lastobs),numeth,lambda,0,1,[0 0],0); 
            else
                if estoption == 0
                    % calculate likelihood only (no estimation)
                    % use parameters for the specific country
                    % check insider constraints, get model fit data
                    [llarray(c),constflag(c),sharedata(1:numobs,:,c),members(1:numobs,:,c)] = splitlikelihood(paramvec((c-1)*4+1:(c-1)*4+4)',popshare(firstobs:lastobs),...
                    govshare(firstobs:lastobs),P,countryregimes,includeobs(firstobs:lastobs),numeth,lambda,0,0,[ae(1) ae(2)],1); 
                elseif estoption == 1
                    % run ga algorithm for each country
                    % alpha and epsilon are fixed values
                    myll = @(cparam)splitlikelihood(cparam,popshare(firstobs:lastobs),...
                    govshare(firstobs:lastobs),P,countryregimes,includeobs(firstobs:lastobs),numeth,lambda,0,0,[ae(1) ae(2)],0);
                    [cparamest cfval cexitflag] = ga(myll,4,gaoptions);
                    if cexitflag ~= 1
                        disp('WARNING: GA estimation did not converge for country:');
                        c
                    end;
                    % return likelihood and estimated parameters
                    llarray(c) = cfval;
                    countryparam(c,:) = cparamest;
                else
                    % run fminsearch starting from previous value
                    countryparam_0 = countryparam(c,:);
                    myll = @(cparam)splitlikelihood(cparam,popshare(firstobs:lastobs),...
                    govshare(firstobs:lastobs),P,countryregimes,includeobs(firstobs:lastobs),numeth,lambda,0,0,[ae(1) ae(2)],0);
                    [cparamest,cfval,cfminflag] = fminsearch(myll,countryparam_0,fminopts);
                    if cfminflag ~= 1
                        disp('WARNING: fmincon estimation did not converge for country:');
                        c
                    end;
                    % return likelihood and update country parameters
                    llarray(c) = cfval;
                    countryparam(c,:) = cparamest;
                end;
            end;
        end;
    else
        % no observations to work with
        if vectorize == 1
            llarray(firstobs:lastobs) = 555555; % just some obvious number
        else
            llarray(c,:) = 555555;
        end;
    end;
end;

if vectorize == 1
    % return likelihoods of each observation
    ll = llarray;
else
    % sum country likelihoods
    ll = ones(1,numcountries)*llarray;
end;
        