% This file uses the time periods from 1994-2004 to obtain estimates of the
% parameters of the dynamic coalition model for West African countries.

%
% The optimization procedure proceeds by estimating alpha and epsilon only
% looking at regime transitions.
% It then estimates (using ga and then fmincon) F, gamma, xi, and rprime
% for each country. Then epsilon and rprime are used to recover r for each
% country. Finally, the standard errors are calculated assuming zero
% correlation between the regime parameters (alpha and epsilon) and the
% country parameters.

% Chad Kendall, May 2013

% read in the excel data
% IMPORTANT: It is assumed that the data is sorted from the largest to
% smallest ethinicity within a country/time observation.

clear all;

% set the following to use the top government positions only
usetopshare = 0;

% set the following to skip the estimation and just use the estimates from
% file (for testing subsequent code)
skipest = 0;

% the country parameters are made global so that they can be accessed
% here and within the pooledlikelihood function and so that they maintain
% their values across function calls
% Note: care must be taken to ensure the parameters are the correct values
% for the type of estimation being done
% countryparam is made up of F, gamma, xi, rprime for each country (row)
global countryparam;

% read in data
[rawnumeric, rawtext, other] = xlsread('..\ethnicgroup_sorted.xls','A2:Y11750');

% index of last observation in the data
lastdataobs = 11749;

% include all observations
includeobs = ones(lastdataobs,1);

% create an 3-column array in which the first column is the number of
% ethnicities in the country, the second column is the observation at
% which the country starts in 1994), and the third column is the
% last observation of the country in 2004

countries = [15 21 30 17 10 22 9 16 15 17 10 14 37 20 26; ...
    481 1318 2479 3370 3887 4723 5262 5825 6496 7205 7712 8270 9423 10490 11464; ...
    645 1548 2808 3556 3996 4964 5360 6000 6660 7391 7821 8423 9829 10709 11749]';

% number of countries in the data
numcountries = size(countries,1);

% lambda is the percentage of elites, assumed fixed across countries and
% time
lambda = 1;

% extract the variables over the range of interest
popshare = rawnumeric(1:lastdataobs,5)/100;
leader = rawnumeric(1:lastdataobs,6);
leadertrans = rawnumeric(1:lastdataobs,15);
if usetopshare == 1
    govsize = rawnumeric(1:lastdataobs,10);
    govposition = rawnumeric(1:lastdataobs,11);
else
    govsize = rawnumeric(1:lastdataobs,7);
    govposition = rawnumeric(1:lastdataobs,8);
end;

% calculate share of positions
govshare = govposition./govsize;

% normalize population
P=1;

% determine regimes for each country for the period 1994-2004
% first column is leader, second is length of leadership, third column
% is observation in which regime change occurred (relative to start of
% country)
% here, it is a 3d array in which the 3rd dimension is the country

% estimation is done for Benin, Cameroon, Cote d'Ivoire, Gabon, Guinea,
% Togo
for c=1:numcountries
    
    idx = 1; % increments for each new regime

    % loop over all data in the country looking for leaders and regime changes
    % NOTE: in the case of multiple leaders, only the first will be picked up

    % number of observations for the country
    numobs = countries(c,3) - countries(c,2) + 1;
    
    % first and last observations for the country
    firstobs = countries(c,2);
    lastobs = countries(c,3);
    % number of ethnicities for the country
    numeth = countries(c,1);
    
    regimechange = 1; % start by assuming a change
    startofregime = 1; % start of first regime
    regimes(1,1,c) = NaN; % used to check that leader is found
    regimes(1,3,c) = 1; % first transition associated with first observation
    for i=(1:numobs)
        % i is offset from start of country so need actual row of
        % observation
        row = firstobs + i - 1;
        
        % look for transition only every numeth rows
        if (regimechange == 0 && leadertrans(row) == 1 && mod(i,numeth) == 1)
            regimechange = 1;
            regimes(idx,2,c) = (i-startofregime)/numeth;
            startofregime = i;
            idx = idx + 1;
            regimes(idx,1,c) = NaN; % used to check that leader is found
            regimes(idx,3,c) = i; % observation at which transition occurred
        end;
        if regimechange == 1
            % looking for new leader
            if leader(row) == 1
                % found leader of new regime
                regimes(idx,1,c) = mod(i,numeth);
                if regimes(idx,1,c) == 0
                    regimes(idx,1,c) = numeth;
                end;
                regimechange = 0;
            end;
        end;

    end;

    % length of last regime
    % regimechange should be zero as we should have found the last start of
    % regime and then the leader of that regime
    if regimechange == 0 
        regimes(idx,2,c) = (numobs+1-startofregime)/numeth;
    else
        error('A leader was not found in the last regime.');
    end;

    % check that leaders were always found
    if (sum(isnan(regimes(:,1,c))))
         error('A leader was not found for at least one regime.');
    end;
end;

% the following estimates alpha and epsilon only

% initial values
alpha_0 = 0;
epsilon_0 = 0;
myll = @(paramvec)pooledlikelihood(paramvec,popshare,govshare,P,countries,regimes,includeobs,lambda,0,1,0,[]);
fminopts = optimset('Display','off','TolX',1e-6,'TolFun',1e-8, 'MaxIter', 1000,'MaxFunEvals',10000);
[aeest,ll,fminflag] = fminsearch(myll,[alpha_0 epsilon_0],fminopts);

% calculate the covariance matrix of alpha and epsilon for use in obtaining
% standard errors later

varae = OPG('pooledlikelihood',aeest',popshare,govshare,P,countries,regimes,includeobs,lambda,1,1,0,[]);

% now proceed with full estimation using the above estimates for alpha and
% epsilon

% within the pooled likelihood function, given the values of alpha and
% epsilon provided, the country parameters are estimated for each country:
%   Fprime - leadership payoff, transformed
%   gamma_t - probability of successful coup (transformed)
%   xi - parameter governing (beta) error distribution
%   rprime - the profit to cost ratio for a revolution (per elite member)

% perform a first estimation using previously determined estimates
% for alpha and epsilon and running ga for each country

% first load existing estimates if available to place into initial
% population, ensuring likelihood is at least as good as previous run

if usetopshare == 1
    paramfile = 'toppost1994param.mat';
else
    paramfile = 'post1994param.mat';
end

fileexists = exist(paramfile);
if fileexists ~=0
    load(paramfile,'countryparam');
    disp('Loaded previous parameter values to use as initial population for ga.');
end;

if skipest == 0 % perform estimation
    % all of the actual estimation is done with the pooledlikelihood function
    llinit = pooledlikelihood([],popshare,govshare,P,countries,regimes,includeobs,lambda,0,0,1,[aeest(1) aeest(2)]);
    % report likelihood and country estimates
    disp('After ga, the likelihood and country estimates are:');
    llinit
    countryparam

    % now run fminsearch for each country starting from values obtained
    % with ga (using same alpha and epsilon again)
    llfinal = pooledlikelihood(aeest,popshare,govshare,P,countries,regimes,includeobs,lambda,0,0,2,[aeest(1) aeest(2)]);

    % save the parameters so that they can be placed into the initial
    % population in the next run
    save(paramfile,'countryparam');
end;

% copy all country parameters into a single row vector
for c=1:numcountries
    fullparam((c-1)*4+1:(c-1)*4+4) = countryparam(c,:);
end;

% report likelihood, check insider revolution constraints, and get model
% fit information and leadership premium
disp('After fminsearch, the likelihood is:');
[ll,llarray,constflag] = pooledlikelihood(fullparam',popshare,govshare,P,countries,regimes,includeobs,lambda,0,0,0,[aeest(1); aeest(2)]);
ll
% if any insider constraint is violated, report an error
if sum(constflag) > 0
    constflag
    disp('Error: A one above indicates that the insider revolution constraint was violated for that country.');
end;

% calculate standard errors using the outer product of gradients
% likelihood function returns the vector of likelihoods rather than total
% this is the variance for the country parameters only: the covariance
% between alpha and epsilon is assumed zero
varcountry = OPG('pooledlikelihood',fullparam',popshare,govshare,P,countries,regimes,includeobs,lambda,1,0,0,[aeest(1) aeest(2)]);

% create a variance-covariance matrix for all parameters
varfull = zeros(4*numcountries+2,4*numcountries+2);
varfull(1:2,1:2) = varae;
varfull(3:4*numcountries+2,3:4*numcountries+2) = varcountry;

% calculate r,gamma, and F for each country
epsilon = abs(aeest(2))/(1+abs(aeest(2)));
delta = 0.95; % calibrated

rarray=zeros(numcountries,1);
Farray=zeros(numcountries,1);
gammaarray=zeros(numcountries,1);
for c=1:numcountries
    rarray(c) = delta*epsilon/(fullparam(4*c)*(1-delta)+delta*epsilon);
    gammaarray(c) = abs(fullparam(4*(c-1)+2))/(1+abs(fullparam(4*(c-1)+2)));
    Farray(c) = fullparam(4*(c-1)+1)*(1-gammaarray(c))/gammaarray(c);
end;

% now create a matrix for applying the delta method
% each row corresponds to one parameter to be determined as a function of
% the estimated parameters: F,r, epsilon, and gamma require transformations

J = zeros(4*numcountries+2,4*numcountries+2);
J(1,1) = 1; % no transformation for alpha
J(2,2) = transformder(aeest(2)); % derivative of transformation for epsilon
for c=1:numcountries
    J(4*(c-1)+3,4*(c-1)+3) = (1-gammaarray(c))/gammaarray(c); % derivative of F wrt F'
    J(4*(c-1)+3,4*(c-1)+4) = -fullparam(4*(c-1)+1)/gammaarray(c)^2*transformder(fullparam(4*(c-1)+2)); % derivative of F wrt gamma
    J(4*(c-1)+4,4*(c-1)+4) = transformder(fullparam(4*(c-1)+2)); % derivative of transformation for gamma
    J(4*(c-1)+5,4*(c-1)+5) = 1; % no transformation for xi
    J(4*(c-1)+6,4*(c-1)+6) = -rarray(c)^2*(1-delta)/delta/epsilon; % derivative of r wrt rprime
    J(4*(c-1)+6,2) = rarray(c)*(1-rarray(c))*transformder(aeest(2))/epsilon;% derivative of r wrt to epsilon_t
end;

seorig = sqrt(diag(inv(varfull))); % untransformed standard errors
se = sqrt(diag(J*inv(varfull)*J')); % transformed standard errors

% report estimates
disp('The estimated parameters (w/ std errors below) are: alpha, epsilon');
[aeest(1) epsilon]
se(1:2)
disp('F, gamma, xi, r, Fprime, rprime, negLL (per country):');
for c=1:numcountries
    % report for a country
    [Farray(c) gammaarray(c) fullparam(4*(c-1)+3) rarray(c) fullparam(4*(c-1)+1)  fullparam(4*(c-1)+4) llarray(c)]
    [se(4*(c-1)+3) se(4*(c-1)+4) se(4*(c-1)+5) se(4*(c-1)+6) seorig(4*(c-1)+3) seorig(4*(c-1)+6)]
end
