% This file uses the time periods from 1960-1993 to obtain estimates of the
% parameters of the dynamic coalition model for West African countries.
% It then uses these estimates to make out-of-sample predictions for 1994-2004
% looking at what would have happened had France stayed involved in these
% countries. All countries are estimated even though we are only interested
% in the West African ones.
%
% The optimization procedure proceeds by estimating alpha and epsilon only
% looking at regime transitions.
% It then estimates (using ga and then fmincon) F, gamma, xi, and rprime
% for each country. Then epsilon and rprime are used to recover r for each
% country. Finally, the standard errors are calculated assuming zero
% correlation between the regime parameters (alpha and epsilon) and the
% country parameters.

% Chad Kendall, May 2013

% read in the excel data
% IMPORTANT: It is assumed that the data is sorted from the largest to
% smallest ethinicity within a country/time observation.

clear all;

% set the following to use the top government positions only
usetopshare = 0;

% set the following to skip the estimation and just use the estimates from
% file (for testing subsequent code)
skipest = 0;

% the country parameters are made global so that they can be accessed
% here and within the pooledlikelihood function and so that they maintain
% their values across function calls
% Note: care must be taken to ensure the parameters are the correct values
% for the type of estimation being done
% countryparam is made up of F, gamma, xi, rprime for each country (row)
global countryparam;

% read in data
[rawnumeric, rawtext, other] = xlsread('..\ethnicgroup_sorted.xls','A2:Y11750');

% index of last observation in the data
lastdataobs = 11749;

% include all observations
includeobs = ones(lastdataobs,1);

% create an 3-column array in which the first column is the number of
% ethnicities in the country, the second column is the observation at
% which the country starts in the data (1960), and the third column is the
% last observation of the country in 1993
% note that different countries are observed in different years, so it is
% not possible to calculate the location of the start of 1994 directly

countries = [15 21 30 17 10 22 9 16 15 17 10 14 37 20 26; ...
    1 646 1549 2809 3557 3997 4965 5361 6001 6661 7392 7822 8424 9830 10710; ...
    480 1317 2478 3369 3886 4722 5261 5824 6495 7204 7711 8269 9422 10489 11463]';
% cutoff is 1994 instead of 1993
%    495 1338 2518 3386 3896 4744 5270 5840 6510 7221 7721 8283 9459 10509 11489]';

% number of countries in the data
numcountries = size(countries,1);

% lambda is the percentage of elites, assumed fixed across countries and
% time
lambda = 1;

% extract the variables over the range of interest
popshare = rawnumeric(1:lastdataobs,5)/100;
leader = rawnumeric(1:lastdataobs,6);
leadertrans = rawnumeric(1:lastdataobs,15);
if usetopshare == 1
    govsize = rawnumeric(1:lastdataobs,10);
    govposition = rawnumeric(1:lastdataobs,11);
else
    govsize = rawnumeric(1:lastdataobs,7);
    govposition = rawnumeric(1:lastdataobs,8);
end;

% calculate share of positions
govshare = govposition./govsize;

% normalize population
P=1;

% determine regimes for each country for the period 1960-1993
% first column is leader, second is length of leadership, third column
% is observation in which regime change occurred (relative to start of
% country)
% here, it is a 3d array in which the 3rd dimension is the country

% estimation is done for Benin, Cameroon, Cote d'Ivoire, Gabon, Guinea,
% Togo
for c=1:numcountries
    
    idx = 1; % increments for each new regime

    % loop over all data in the country looking for leaders and regime changes
    % NOTE: in the case of multiple leaders, only the first will be picked up

    % number of observations for the country
    numobs = countries(c,3) - countries(c,2) + 1;
    
    % first and last observations for the country
    firstobs = countries(c,2);
    lastobs = countries(c,3);
    % number of ethnicities for the country
    numeth = countries(c,1);
    
    regimechange = 1; % start by assuming a change
    startofregime = 1; % start of first regime
    regimes(1,1,c) = NaN; % used to check that leader is found
    regimes(1,3,c) = 1; % first transition associated with first observation
    for i=(1:numobs)
        % i is offset from start of country so need actual row of
        % observation
        row = firstobs + i - 1;
        
        % look for transition only every numeth rows
        if (regimechange == 0 && leadertrans(row) == 1 && mod(i,numeth) == 1)
            regimechange = 1;
            regimes(idx,2,c) = (i-startofregime)/numeth;
            startofregime = i;
            idx = idx + 1;
            regimes(idx,1,c) = NaN; % used to check that leader is found
            regimes(idx,3,c) = i; % observation at which transition occurred
        end;
        if regimechange == 1
            % looking for new leader
            if leader(row) == 1
                % found leader of new regime
                regimes(idx,1,c) = mod(i,numeth);
                if regimes(idx,1,c) == 0
                    regimes(idx,1,c) = numeth;
                end;
                regimechange = 0;
            end;
        end;

    end;

    % length of last regime
    % regimechange should be zero as we should have found the last start of
    % regime and then the leader of that regime
    if regimechange == 0 
        regimes(idx,2,c) = (numobs+1-startofregime)/numeth;
    else
        error('A leader was not found in the last regime.');
    end;

    % check that leaders were always found
    if (sum(isnan(regimes(:,1,c))))
         error('A leader was not found for at least one regime.');
    end;
end;

% the following estimates alpha and epsilon only

% initial values
alpha_0 = 0;
epsilon_0 = 0;
myll = @(paramvec)pooledlikelihood(paramvec,popshare,govshare,P,countries,regimes,includeobs,lambda,0,1,0,[]);
fminopts = optimset('Display','off','TolX',1e-6,'TolFun',1e-8, 'MaxIter', 1000,'MaxFunEvals',10000);
[aeest,ll,fminflag] = fminsearch(myll,[alpha_0 epsilon_0],fminopts);

% calculate the covariance matrix of alpha and epsilon for use in obtaining
% standard errors later

varae = OPG('pooledlikelihood',aeest',popshare,govshare,P,countries,regimes,includeobs,lambda,1,1,0,[]);

% now proceed with full estimation using the above estimates for alpha and
% epsilon

% within the pooled likelihood function, given the values of alpha and
% epsilon provided, the country parameters are estimated for each country:
%   Fprime - leadership payoff, transformed
%   gamma_t - probability of successful coup (transformed)
%   xi - parameter governing (beta) error distribution
%   rprime - the profit to cost ratio for a revolution (per elite member)

% perform a first estimation using previously determined estimates
% for alpha and epsilon and running ga for each country

% first load existing estimates if available to place into initial
% population, ensuring likelihood is at least as good as previous run

if usetopshare == 1
    paramfile = 'toppre1994param.mat';
else
    paramfile = 'pre1994param.mat';
end

fileexists = exist(paramfile);
if fileexists ~=0
    load(paramfile,'countryparam');
    disp('Loaded previous parameter values to use as initial population for ga.');
end;

if skipest == 0 % perform estimation
    % all of the actual estimation is done with the pooledlikelihood function
    llinit = pooledlikelihood([],popshare,govshare,P,countries,regimes,includeobs,lambda,0,0,1,[aeest(1) aeest(2)]);
    % report likelihood and country estimates
    disp('After ga, the likelihood and country estimates are:');
    llinit
    countryparam

    % now run fminsearch for each country starting from values obtained
    % with ga (using same alpha and epsilon again)
    llfinal = pooledlikelihood(aeest,popshare,govshare,P,countries,regimes,includeobs,lambda,0,0,2,[aeest(1) aeest(2)]);

    % save the parameters so that they can be placed into the initial
    % population in the next run
    save(paramfile,'countryparam');
end;

% copy all country parameters into a single row vector
for c=1:numcountries
    fullparam((c-1)*4+1:(c-1)*4+4) = countryparam(c,:);
end;

% report likelihood, check insider revolution constraints, and get model
% fit information and leadership premium
disp('After fminsearch, the likelihood is:');
[ll,llarray,constflag] = pooledlikelihood(fullparam',popshare,govshare,P,countries,regimes,includeobs,lambda,0,0,0,[aeest(1); aeest(2)]);
ll
% if any insider constraint is violated, report an error
if sum(constflag) > 0
    constflag
    disp('Error: A one above indicates that the insider revolution constraint was violated for that country.');
end;

% calculate standard errors using the outer product of gradients
% likelihood function returns the vector of likelihoods rather than total
% this is the variance for the country parameters only: the covariance
% between alpha and epsilon is assumed zero
varcountry = OPG('pooledlikelihood',fullparam',popshare,govshare,P,countries,regimes,includeobs,lambda,1,0,0,[aeest(1) aeest(2)]);

% create a variance-covariance matrix for all parameters
varfull = zeros(4*numcountries+2,4*numcountries+2);
varfull(1:2,1:2) = varae;
varfull(3:4*numcountries+2,3:4*numcountries+2) = varcountry;

% calculate r,gamma, and F for each country
epsilon = abs(aeest(2))/(1+abs(aeest(2)));
delta = 0.95; % calibrated

rarray=zeros(numcountries,1);
Farray=zeros(numcountries,1);
gammaarray=zeros(numcountries,1);
for c=1:numcountries
    rarray(c) = delta*epsilon/(fullparam(4*c)*(1-delta)+delta*epsilon);
    gammaarray(c) = abs(fullparam(4*(c-1)+2))/(1+abs(fullparam(4*(c-1)+2)));
    Farray(c) = fullparam(4*(c-1)+1)*(1-gammaarray(c))/gammaarray(c);
end;

% now create a matrix for applying the delta method
% each row corresponds to one parameter to be determined as a function of
% the estimated parameters: F,r, epsilon, and gamma require transformations

J = zeros(4*numcountries+2,4*numcountries+2);
J(1,1) = 1; % no transformation for alpha
J(2,2) = transformder(aeest(2)); % derivative of transformation for epsilon
for c=1:numcountries
    J(4*(c-1)+3,4*(c-1)+3) = (1-gammaarray(c))/gammaarray(c); % derivative of F wrt F'
    J(4*(c-1)+3,4*(c-1)+4) = -fullparam(4*(c-1)+1)/gammaarray(c)^2*transformder(fullparam(4*(c-1)+2)); % derivative of F wrt gamma
    J(4*(c-1)+4,4*(c-1)+4) = transformder(fullparam(4*(c-1)+2)); % derivative of transformation for gamma
    J(4*(c-1)+5,4*(c-1)+5) = 1; % no transformation for xi
    J(4*(c-1)+6,4*(c-1)+6) = -rarray(c)^2*(1-delta)/delta/epsilon; % derivative of r wrt rprime
    J(4*(c-1)+6,2) = rarray(c)*(1-rarray(c))*transformder(aeest(2))/epsilon;% derivative of r wrt to epsilon_t
end;

seorig = sqrt(diag(inv(varfull))); % untransformed standard errors
se = sqrt(diag(J*inv(varfull)*J')); % transformed standard errors

% report estimates
disp('The estimated parameters (w/ std errors below) are: alpha, epsilon');
[aeest(1) epsilon]
se(1:2)
disp('F, gamma, xi, r, Fprime, rprime, negLL (per country):');
for c=1:numcountries
    % report for a country
    [Farray(c) gammaarray(c) fullparam(4*(c-1)+3) rarray(c) fullparam(4*(c-1)+1)  fullparam(4*(c-1)+4) llarray(c)]
    [se(4*(c-1)+3) se(4*(c-1)+4) se(4*(c-1)+5) se(4*(c-1)+6) seorig(4*(c-1)+3) seorig(4*(c-1)+6)]
end

% this check no longer seems to be necessary as repeated estimation
% converges to corners automatically
% % as a double-check that the estimates are in fact optimal, check the
% % extreme points of gamma (0 and 1), using the estimated value of all other
% % parameters: calculate the loglikelihood for these extreme points
% 
% g0param = fullparam; % copy of parameters for gamma = 0
% g1param = fullparam; % copy of parameters for gamma = 1a = 1
% 
% % replace gamma with 0 or 1
% % actually use values slightly away from 0 and 1 because model is not
% % strictly valid at these extremes
% for c=1:numcountries
%     g0param(4*(c-1)+2) = 1e-7/(1-1e-7); % transformed 1e-7
%     g1param(4*(c-1)+2) = (1-1e-7)/(1e-7); %transformed 1-1e-7
% end;
% 
% % calculate likelihoods
% [ll,llg0,constflag] = pooledlikelihood([aeest(1); aeest(2); g0param'],popshare,govshare,P,countries,regimes,lambda,0,0,0,[aeest(1); aeest(2)]);
% [ll,llg1,constflag] = pooledlikelihood([aeest(1); aeest(2); g1param'],popshare,govshare,P,countries,regimes,lambda,0,0,0,[aeest(1); aeest(2)]);
% 
% disp('The likelihoods for each country with gamma=0 are:');
% llg0'
% disp('The likelihoods for each country with gamma=1 are:');
% llg1'

% now proceed to predict out-of-sample for 1981-2004 and perform
% counterfactuals
% rather than identifying regimes first, we just look at each country in
% each time period and predict the shares for that period for the estimated
% parameters and counterfactuals

% 4 sets of counterfactuals are performed
% in the first 3, one of the parameters is scaled according to:
Fscale = 0.75;
gammascale = 0.75;
rscale = 0.9;
% in the 4th, the ethnic fractionalization is increased by adding 1% to
% each population share below the median and subtracting 1% from each share
% above the median

% also create sharedata
%   first column is predicted shares for leaders (zeros for others)
%   second column is observed shares for leaders (zeros for others)
%   third column is predicted shares for largest (zeros for others)
%   fourth column is observed shares for largest (zeros for others)
% and members:
%   first column of members is predicted share > 0
%   second column of members is observed share > 0
% and fullusepopshare:
%   the population share for each country in each time period allowing for 
%       increased ethnic fractionalization

sharedata = zeros(size(popshare,1),4); % initialization
members = zeros(size(popshare,1),2); % initialization
fullusepopshare = zeros(size(popshare,1),1); % initialization

disp('Model fit and counterfactual data for 1994-2004:');

% loop over countries
for c = 1:numcountries
    
    disp('Data is for country:');
    c
    
    % number of observations for the country (1994-2004)
    if c == numcountries
        numobs = lastdataobs - countries(c,3);
    else
        numobs = countries(c+1,2) - countries(c,3) - 1;
    end;
    
    % first and last observations for the country
    firstobs = countries(c,3)+1;
    lastobs = firstobs+numobs-1;
    
    % number of ethnicities for the country
    numeth = countries(c,1);
    
    % calculate number of time periods (differs by country)
    numperiods = numobs/countries(c,1);
    
    % loop over each set of parameters
%     for p=1:5
        
        % assign parameters (only F, gamma, and rprime are used)
%         if (p == 1) % estimated
            Fprime = fullparam(4*(c-1)+1);
            gamma_t = fullparam(4*(c-1)+2);
            rprime = fullparam(4*(c-1)+4);
            
            gamma = abs(gamma_t)/(1+abs(gamma_t));
            F = Fprime*(1-gamma)/gamma;

            disp('Model fit for estimated data:');
%         elseif p == 2 % scale F
%             Fprime = fullparam(4*(c-1)+1);
%             gamma_t = fullparam(4*(c-1)+2);
%             rprime = fullparam(4*(c-1)+4);
%                         
%             gamma = abs(gamma_t)/(1+abs(gamma_t));
%             F = Fscale*Fprime*(1-gamma)/gamma;
% 
%             disp('Model fit for counterfactual with F scaled:');
%         elseif p == 3 % scale gamma
%             Fprime = fullparam(4*(c-1)+1);
%             gamma_t = fullparam(4*(c-1)+2);
%             rprime = fullparam(4*(c-1)+4);
%             
%             gamma = abs(gamma_t)/(1+abs(gamma_t));
%             F = Fprime*(1-gamma)/gamma;
% 
%             gamma = gamma*gammascale;
% 
%             disp('Model fit for counterfactual with gamma scaled:');
%         elseif p == 4 % scale r
%             Fprime = fullparam(4*(c-1)+1);
%             gamma_t = fullparam(4*(c-1)+2);
%             rprime = fullparam(4*(c-1)+4)/rscale+delta*epsilon*(1-rscale)/(1-delta)/rscale;
%             
%             gamma = abs(gamma_t)/(1+abs(gamma_t));
%             F = Fprime*(1-gamma)/gamma;
% 
%             disp('Model fit for counterfactual with r scaled:');
%         else % increased fractionalization
%             Fprime = fullparam(4*(c-1)+1);
%             gamma_t = fullparam(4*(c-1)+2);
%             rprime = fullparam(4*(c-1)+4);
%             
%             gamma = abs(gamma_t)/(1+abs(gamma_t));
%             F = Fprime*(1-gamma)/gamma;
% 
%             disp('Model fit for counterfactual with increased fractionalization:');
%         end;    
        
        usepopshare = zeros(numeth,1); % initialization
        % increase ethnic fractionalization by modifying population shares
        % extract population shares for the country
        temppopshare = popshare(firstobs:firstobs+numeth-1);
%         if p == 5
%             % find median population share
%             medpopshare = median(temppopshare);
%             % loop over population shares to add 1% to those smaller than
%             % median and subtract 1% from those larger
%             for n=1:numeth
%                 if temppopshare(n) > medpopshare
%                     usepopshare(n) = temppopshare(n) - 0.01;
%                 elseif temppopshare(n) == medpopshare
%                     usepopshare(n) = temppopshare(n);
%                 else
%                     usepopshare(n) = temppopshare(n) + 0.01;
%                 end;
%             end;
%         else
            % for other counterfactuals, use actual population shares
            usepopshare = temppopshare;
%        end;
            
        % loop over time periods
        for t = 1:numperiods

            % copy usepopshare into each time period
            fullusepopshare(firstobs+(t-1)*numeth:firstobs+t*numeth-1) = usepopshare;
            
            % loop over observations within the time period to find the leader
            for j=1:numeth

                % identify row of actual observation
                row = firstobs + (t-1)*numeth + j-1;

                if leader(row) == 1
                    % found leader (first one, if multiple)
                    currentleader = j;
                    break;
                end;
            end;

            % find splitter
            [splitter,corecoal,bigoutsiders] = splitgroup(usepopshare,rprime);
            
            if splitter == 0
                % couldn't find a splitter, due to scaling of rprime
                if rprime >= 1
                    % empty coalition
                    splitter = 1;
                    corecoal = zeros(numeth,1);
                    % all groups are big enough to fill the (non-positive) gap
                    bigoutsiders = ones(numeth,1);
                    bigoutsiders(1) = 0;                    
                    disp('Could not find a splitter because rprime > 1.');
                else
                    error('rprime < 0 which should never happen!');
                end;
            end;

            % calculate shares given the leader
            [shares,excl,flag,actualsplitter,actualsplitsize] = splitpredictedshares(F,gamma, ...
                rprime,splitter,corecoal,bigoutsiders,usepopshare,currentleader,P,lambda);
            if flag == 1
                disp('Warning predicted shares were negative for country:');
                c
            end;
            
            % check insider revolution constraint
 
            % calculate elite sizes
            elite = lambda*P.*usepopshare;
    
            % need to check all non-leader members of the coalition
            % so start by identifying non-leader coalition members
            coalvec = (shares > 0); % all members including leader

            % now remove leader because we won't check their insider
            % constraint
            coalvec(currentleader) = 0;

            % loop over all groups to check each member's constraint
            for m=1:numeth
                if coalvec(m) == 1;

                    % set the size of the group to its elite size if
                    % included whole or the split size if it is split

                    if m == actualsplitter
                        size_m = actualsplitsize;
                    else
                        size_m = elite(m);
                    end;

                    % get value when not leader
                    x_m = shares(m)/size_m;

                    % now find shares that member would get if leader
                    [tempshares,tempexcl,tempflag] = splitpredictedshares(F,gamma,rprime, ...
                        splitter,corecoal,bigoutsiders,usepopshare,m,P,lambda);
                    if tempflag == 1
                        disp('WARNING: Some shares were negative when calculating the shares with coalition member being leader!');
                        disp('Shares:');
                        tempshares'
                    end;

                    % value when leader always uses full group size
                    xbar_m = tempshares(m)/elite(m);

                    % here we are checking a sufficient condition such
                    % that if it holds the insider constraint is always
                    % satisfied

                    scaledbar_m = size_m*delta*epsilon/(rprime*(1-delta)+delta*epsilon)*xbar_m;

                    if scaledbar_m > x_m
                        disp('The insider revolution constraint is violated for country, c, at time, t:')
                        [c t]
                    end;         
                end;
            end;

            % model fit data
            sharedata(firstobs+(t-1)*numeth+currentleader-1,1) = ...
                shares(currentleader); % predicted leader shares
            sharedata(firstobs+(t-1)*numeth+currentleader-1,2) = ...
                govshare(firstobs+(t-1)*numeth+currentleader-1); % observed leader shares
            sharedata(firstobs+(t-1)*numeth,3) = ...
                shares(1); % predicted largest shares
            sharedata(firstobs+(t-1)*numeth,4) = ...
                govshare(firstobs+(t-1)*numeth); % observed largest shares
            members(firstobs+(t-1)*numeth:firstobs+t*numeth-1,1) = ...
                (shares > 0); % predicted member
            members(firstobs+(t-1)*numeth:firstobs+t*numeth-1,2) = ...
                (govshare(firstobs+(t-1)*numeth:firstobs+t*numeth-1) > 0); % observed member
        end;

        % calculate model fit or counterfactual data
        
        disp('Displaying pred/obs leader share, pred/obs largest share, pred/obs coal, in/out correct:');

        % calculate average leadership and largest shares (predicted vs. observed)
        avgldrpredshare = sum(sharedata(firstobs:lastobs,1))/numperiods;
        avgldrobsshare = sum(sharedata(firstobs:lastobs,2))/numperiods;
        avglrgpredshare = sum(sharedata(firstobs:lastobs,3))/numperiods;
        avglrgobsshare = sum(sharedata(firstobs:lastobs,4))/numperiods;

        % calculate average coalition sizes weighted by pop'n
        avgwtpredcoalsize = sum(members(firstobs:lastobs,1).*fullusepopshare(firstobs:lastobs))/numperiods;
        avgwtobscoalsize = sum(members(firstobs:lastobs,2).*fullusepopshare(firstobs:lastobs))/numperiods;

        % calculate average correct predictions weighted by pop'n
        avgwtcorrectin = sum(fullusepopshare(firstobs:lastobs).*members(firstobs:lastobs,1).*members(firstobs:lastobs,2))/numperiods;
        avgwtcorrectout = sum(fullusepopshare(firstobs:lastobs).*(1-members(firstobs:lastobs,1)).*(1-members(firstobs:lastobs,2)))/numperiods;
        
        % output
        [avgldrpredshare avgldrobsshare avglrgpredshare avglrgobsshare avgwtpredcoalsize avgwtobscoalsize ...
        avgwtcorrectin avgwtcorrectout]
%    end;
end;

