function [ll] = simplelikelihood(xi,popshare,govshare,regimes,includeobs,numeth,option,vectorize)

% This function calculates the (negative) log likelihood of 3 simple
% models. In the random model, each coalition is formed by randomly
% selecting people from the population. This means that the predicted share
% for an ethnicity is simply equal to its share of the population. In the
% second model, the leader keeps all. In the third, shares depend on the 
% relative sizes of the ethnic groups (polarization).
%
% Inputs:
%
% xi: this is the only parameter to be estimated: it controls the shape of
% the beta distribution of the error terms
%
% popshare: vector of the population share for each ethnicity-time obs
%
% govshare: observed share of government positions for each ethnicity-time
% obs
%
% regimes: three column vector where first column is leader, second
% column is length of leadership, third column is observation in which
% transition occurred; one row per regime
%
% includeobs : observations to be included in the likelihood
%
% numeth: number of ethnicities (assumed fixed over time), used to delineate
% time periods
%
% option : 0: random model
%          1: leader's group gets entire share
%          2: relative polarization model
%
% vectorize : set to 1 to return individual log-likelihoods instead of sum

predshare = zeros(size(popshare,1),1); % initialization
if option == 0
    % predicted shares are just proportion of population
    predshare = popshare;
elseif option == 1
    % set leader's share to 1, leave others at zero
    % loop over regimes
    for r=1:size(regimes,1)
        % loop over time periods within regime
        for t=1:regimes(r,2)
            % set leader's share to 1
            predshare((regimes(r,3)+(t-1)*numeth+regimes(r,1))-1,1) = 1;
        end;
    end;
else 
    % shares are set based on relative sizes
    for t = 1:size(popshare,1)/numeth % number of time periods
        for i=1:numeth
            if i==1
                predshare((t-1)*numeth+i) = 1;
            else
                predshare((t-1)*numeth+i) = popshare((t-1)*numeth+i)/popshare((t-1)*numeth+i-1);
            end;
        end;
    end;
    % normalize to sum shares to 1
    predshare = predshare/sum(predshare(1:numeth));
end;

% identify those observed in the coalition
incoalition = (govshare>0);

% create vector of groups to exclude (smallest group)
nobs = size(popshare,1);
numperiods = nobs/numeth;
exclude = zeros(nobs,1); % initialization
for t=1:numperiods
    exclude(t*numeth)=1;
end

% calculate beta distribution probabilities for those in and out of the
% coalition
% because the support of the distribution we want, we need to scale the pdf
% and cdf as follows
% let y be the r.v. with support [-1,1] and we want to create x with
% support [0,1] such that x is distributed with a beta distribution
% we let x=(y+1)/2 and then the pdf of y is 1/2beta((y+1)/2) and the cdf of
% y is B((y+1)/2)
inpr = 1/2*betapdf((govshare-predshare+1)/2,xi,xi);
outpr = betacdf((1-predshare)/2,xi,xi);

% create a vector of each observation's contribution to the log-likelihood
% only consider non-excluded groups when adding to the likelihood
% don't let log go to negative infinity as it makes whole expression
% undefined even when that part of the expression is not used
llobs = includeobs.*(1-exclude).*(incoalition.*max(-1E5,log(inpr))+(1-incoalition).*max(-1E5,log(outpr)));

if vectorize == 1
    % return individual likelihoods
    ll = -llobs;
else
    % sum up likelihoods
    ll = -ones(1,size(popshare,1))*llobs;         
end;

