function [splitter, corecoal, bigoutsiders] = splitgroup(popshare,rprime)

% This function finds the splitter group, j, given an ordered list of population
% shares (biggest to smallest). The group to be split is the one such that
% if all smaller groups are outsiders, a revolution will not occur, but if
% the splitter group is also excluded, a revolution will occur. In this way, a
% leader must include part of the splitter group to avoid a revolution.

% The function returns 0 if no such group exists.
% The function also returns two vectors the size of the number of ethnicities:
%   corecoal : core coalition members (up to j-2) are set to 1, others 0
%   bigoutsiders : members smaller than  j that are big enough to fill
%   the gap are set to 1, others are 0 (the gap is the extra pop'n share
%   needed to ensure no revolution occurs)
% Both vectors are only valid when a stable coalition has been found (splitter
% is not zero).

% It takes as inputs:
%
% popshare - share of population (vector of size equal to number of ethnicities) 
% rprime = delta*epsilon*(1-r)/r/(1-delta) where delta is the
%    discount factor, epsilon is the probability of regime change, and r is
%    the factor by which patronage is destroyed when a revolution occurs

% number of ethnicities
numeth  = size(popshare,1);

splitter = 0; % default to not possible to find a coalition such that no
% revolution occurs

cumshare = popshare(1); % start with biggest group only

% its only possible to find a stable coalition if rprime is within [0,1)
% when rprime = 1, an empty coalition is stable, but this will never match the
% data so we flag it is not possible

if ((rprime < 1) && (rprime >=0))
    for i=1:numeth
    
        if cumshare >= (1 - rprime)
            % have found splitter group
            splitter = i;
            break;
        end;
        if i ~= numeth
            cumshare = cumshare + popshare(i+1);
        else
            % we should never reach here as if 1=numeth cumshare should be
            % 1 and 1-rprime should be <=1; however, for Uganda or Tanzania, the sum of
            % the popshare is not quite 1, so we can hit here if rprime=0
            disp('Warning: did not find a split group normally.');
            splitter = numeth;
        end;
    end;
end;

corecoal = zeros(numeth,1);
bigoutsiders = zeros(numeth,1);

% create vectors of core coalition members and big outsiders
if splitter > 0

    if splitter > 2
        corecoal(1:splitter-2) = 1;
    end;
    
    gap = 1- rprime - (cumshare-popshare(splitter));
    for i = (splitter+1):numeth
         if popshare(i) >= gap
            bigoutsiders(i) = 1;
         end;
    end;
end;