function [ll,constflag,sharedata,members] = splitlikelihood(paramvec,popshare,govshare,P,regimes,includeobs,numeth,lambda,vectorize,estregimes,ae,final)

% This function calculates the (negative) log likelihood of the dynamic
% coalition model.
% It returns either the total log likelihood or a vector of log likelihoods
% for each observation, as well as the predicted shares for each ethnicity
% in each time period.
%
% constflag is set if the insider revolution constraint is violated (only
% checked when final is set to 1)
%
% sharedata is a 6-column array: first column is predicted leader
% shares, second column is observed leader shares; third and fourth columns
% are the same but for the largest ethnicity, not the leader; fifth and sixth
% columns are the predicted and observed shares for all ethnicities
% each column 1-4 is of length popshare where all non-leader shares are 0
% - valid only when final is 1
%
% members is a 3-column array: first column is 1 if the group is
% predicted to be a member of the coalition: second column is 1 if the
% group is actually a member of the coalition; third column is pop'n share
% each column is of size popshare
% - valid only when final is 1
%
% Inputs:
%
% estregimes: set to 1 to estimate the regime-related parameters only and
% set to 0 to estimate country parameters
%
% ae: used when estregimes==0 to pass in alpha (1st entry) and epsilon
% (2nd entry)
%
% paramvec: the vector of parameters to be estimated depends upon estregimes:
%
%   estregimes = 0
%
%       Fprime - leadership payoff, transformed
%       gamma_t - probability of successful coup (transformed)
%       xi - parameter governing (beta) error distribution
%       rprime = delta*epsilon*(1-r)/r/(1-delta) where delta is the
%               discount factor, epsilon is the probability of regime change, and r is
%               the factor by which patronage is destroyed when a revolution occurs
%
%   estregimes = 1
%       alpha - parameter in probability dist'n over leadership
%       epsilon_t - probability of transition event (transformed)
%       
%
% popshare: vector of the population share for each ethnicity-time obs
%
% govshare: observed share of government positions for each ethnicity-time
% obs
%
% P : total country population (fixed across time)
%
% regimes: three column vector where first column is leader, second
% column is length of leadership, third column is observation in which
% transition occurred; one row per regime
%
% includeobs : a '1' for each observation that should be included in
% calculatng the likelihood
%
% numeth: number of ethnicities (assumed fixed over time), used to delineate
% time periods
%
% lambda : percentage of elites in the population (same for all
% ethnicities)
%
% vectorize : set to 1 to return the vector of likelihoods rather than the
% summed likelihood (used for computing standard errors)
%
% final : set to 1 to check insider constraints: should be set only
% after estimation is complete as it is time-consuming and doesn't affect
% the likelihood; upon a violation, constflag is set to 1
% also used to capture model fit data

% extract parameters from parameter vector
% the probability and discount parameters are transformed from variables on the range
% -Inf to Inf to probabilities here

% delta is now being calibrated to 0.95 somewhat arbitrarily since it
% cannot be recovered from the data
delta = 0.95;

if estregimes == 0
    % alpha and epsilon are fixed (passed in)
    alpha = ae(1);
    epsilon = abs(ae(2))/(1+abs(ae(2)));
    Fprime = paramvec(1);
    gamma = abs(paramvec(2))/(1+abs(paramvec(2)));
    xi = paramvec(3);
    rprime = paramvec(4);
else
    % estimate regime-related parameters only
    % here, F, gamma, xi, and rprime play no role since we estimate alpha and
    % epsilon from the leader transitions only
    alpha = paramvec(1);
    epsilon = abs(paramvec(2))/(1+abs(paramvec(2)));
end;

% Fprime = F*gamma/(1-gamma)

if estregimes == 0
    F = (1-gamma)*Fprime/gamma;
end;

% GENERAL PROCEDURE (estregimes == 0):
%
% the split group,j, is first identified (once) since it only depends on
% rprime and not on the leader
% then, for each regime, the predicted shares are calculated for the
% particulat leader and a vector of predicted shares for all time is
% created
% then, the likelihood is calculated only if valid (non-negative) shares
% were found in each time period
% the smallest group is excluded when determining the likelihood as not all
% shares are independent

% find the splitter group, j
% corecoal has one's for each core coalition member
% bigoutsiders has one's set for each non-core member that is big enough to
% fill the gap and prevent a revolution

if estregimes == 0    
    [splitter,corecoal,bigoutsiders] = splitgroup(popshare(1:numeth),rprime);
end;

constflag = 0; %initialization
sharedata = zeros(size(popshare,1),6); % initialization
leader = zeros(size(popshare,1),1); % initialization
largest = zeros(size(popshare,1),1); % initialization
members = zeros(size(popshare,1),4); % initialization

% proceed only if a standard optimal coalition can be found or estimating
% regime parameters only
if ((estregimes == 0 && splitter ~= 0) || estregimes == 1)
    
    % get the elite sizes (a vector of the sizes of each ethnicity's elite)
    elite = lambda*P.*popshare(1:numeth);

    % calculate vector of exponential terms for probability of leadership
    expelite = exp(alpha.*elite);

    % calculate probability each group becomes leader
    pr = expelite./(ones(1,numeth)*expelite);

    % this is the vector of predicted shares for each ethnicity in each time
    % period
    predshare = zeros(size(popshare,1),1); % initialization
    
    % this is the vector of groups to exclude (i.e. the leader in each time
    % period)
    exclude = zeros(size(popshare,1),1); % initialization

    % this is the contribution to the log likelihood from the regime
    % transition itself
    llregime = zeros(size(regimes,1),1); % initialization
    startregime = 1; % tracks time period of start of regime

    % loop over each regime to calculate
    % 1. the contribution to the likelihood for the regime
    % 2. the predicted shares for the regime
    for i=1:size(regimes,1)
    
         % calculate contribution to likelihood from regime transition
        llregime(i) = log(pr(regimes(i,1))*(1-epsilon)^regimes(i,2)*epsilon);
        
        if estregimes == 0 % estimating all parameters
    
            % calculate predicted shares given the leader of the current regime
            [shares,excl,flag,actualsplitter,actualsplitsize] = splitpredictedshares(F,gamma,rprime,splitter,corecoal,bigoutsiders,popshare(1:numeth),regimes(i,1),P,lambda);
            if flag == 1
                % shares are negative so exit loop
                break;
            end;
                    
            % replicate the predicted shares and exclude vector into a
            % vector for all observations in the regime
            % also create leader and largest vectors (we don't use the
            % original leader from the data because it includes multiple
            % leaders per time period whereas we predict with only one
            % leading ethnicity)
            for j=startregime:(startregime+regimes(i,2)-1)
                predshare((j-1)*numeth+1:j*numeth)=shares;
                % RECENT CHANGE: rather than excluding the splitter as
                % returned by splitpredicted shares, we are instead
                % excluding the smallest group to avoid a discontinuity in the
                % likelihood function when the splitter changes
                %exclude((j-1)*numeth+1:j*numeth)=excl;
                exclude(j*numeth) = 1; % smallest group
                % the following is used to capture model fit (leadership
                % shares) only
                leader((j-1)*numeth+regimes(i,1)) = 1; % leader
                largest((j-1)*numeth+1) = 1; % largest
            end;
            
            % perform a check of the insider constraints
            if final == 1
                
                % need to check all non-leader members of the coalition
                % so start by identifying non-leader coalition members
                coalvec = (shares > 0); % all members including leader
                
                % now remove leader because we won't check their insider
                % constraint
                coalvec(regimes(i,1)) = 0;
                
                % loop over all groups to check each member's constraint
                for m=1:numeth
                    if coalvec(m) == 1;
                    
                        % set the size of the group to its elite size if
                        % included whole or the split size if it is split
                        
                        if m == actualsplitter
                            size_m = actualsplitsize;
                        else
                            size_m = elite(m);
                        end;
                        
                        % get value when not leader
                        x_m = shares(m)/size_m;
                
                        % now find shares that member would get if leader
                        [tempshares,tempexcl,tempflag] = splitpredictedshares(F,gamma,rprime,splitter,corecoal,bigoutsiders,popshare(1:numeth),m,P,lambda);
                        if tempflag == 1
                            disp('WARNING: Some shares were negative when calculating the shares with coalition member being leader!');
                            disp('Shares:');
                            tempshares'
                        end;
                        
                        % value when leader always uses full group size
                        xbar_m = tempshares(m)/elite(m);
                
                        % here we are checking a sufficient condition such
                        % that if it holds the insider constraint is always
                        % satisfied
                        
                        scaledbar_m = size_m*delta*epsilon/(rprime*(1-delta)+delta*epsilon)*xbar_m;
                        
                        if scaledbar_m > x_m
                            constflag = 1;
                        end;         
                    end;
                end;
            end;
        end;
        
        startregime = startregime + regimes(i,2);
    end;  
end;

% proceed to calculate likelihood only when shares are valid for
% each regime or only estimating alpha and epsilon
if ((estregimes == 0 && ((splitter ~= 0 && flag == 0) || (splitter ~=0 && vectorize == 1 ))) || estregimes == 1)
    
    if estregimes == 0
        % create an indicator function which is 1 if an ethnicity has
        % observed positive shares (each time period)

        incoalition = (govshare>0);
        
        % capture model fit data if final is set
        if final == 1
            members(:,1) = (predshare>0); % predicted member
            members(:,2) = incoalition;  % observed member
            % include popshare for weighting
            members(:,3) = popshare;
            sharedata(:,1) = leader.*predshare; % predicted leader share
            sharedata(:,2) = leader.*govshare; % observed leader share
            sharedata(:,3) = largest.*predshare; % predicted largest share
            sharedata(:,4) = largest.*govshare; % observed largest share
            sharedata(:,5) = predshare; % all predicted shares
            sharedata(:,6) = govshare; % all predicted shares
        end;
            
        % calculate beta distribution probabilities for those in and out of the
        % coalition
        % because the support of the distribution we want, we need to scale the pdf
        % and cdf as follows
        % let y be the r.v. with support [-1,1] and we want to create x with
        % support [0,1] such that x is distributed with a beta distribution
        % we let x=(y+1)/2 and then the pdf of y is 1/2beta((y+1)/2) and the cdf of
        % y is B((y+1)/2)
        inpr = 1/2*betapdf((govshare-predshare+1)/2,xi,xi);
        outpr = betacdf((1-predshare)/2,xi,xi);

        % create a vector of each observation's contribution to the log-likelihood
        % only consider non-excluded groups (as identified by exclude) when adding to
        % the likelihood
        % also only include sample of observations specified with
        % includeobs
        llobs = includeobs.*(1-exclude).*(incoalition.*log(inpr)+(1-incoalition).*log(outpr));
    end;

    % don't allow log likelihood to go to negative infinity to prevent
    % optimization issues
    % here, when we create the likelihood of each observation in the case
    % we are estimating all of the parameters, the contribution from the
    % regime is associated with the observation in which the regime changes
    if vectorize == 0
        if estregimes == 0
            ll = -max(-1E5,ones(1,size(popshare,1))*llobs); 
        else
            ll = -max(-1E5,ones(1,size(regimes,1))*llregime); 
        end;
    else
        if estregimes == 0
            ll = -max(-1E5,llobs);
            if flag==1
                disp('WARNING: Shares are negative when calculating likelihood with vectorize==1:');
                shares'
            end;
        else
            ll = -max(-1E5,llregime);
        end;            
    end;
    
else
    % if no optimal coalition with non-negative shares can be found for
    % some regime, set negative of log likelihood to a large value
    if vectorize == 0
        ll = 1E5;
    else
        error('Splitter can not be found when trying to calculate standard errors (vectorize==1).');
    end;
end