function [shares,exclude,flag,actualsplitter,actualsplitsize] = splitpredictedshares(F,gamma,rprime,splitter,corecoal,bigoutsiders,popshare,leader,P,lambda)

% This function calculates the predicted shares x_j*e_j for each ethnicity
% including the leader (which is calculated as 1-sum of other shares).
% It returns the vector of shares for each ethnicity in a given country
% at a given time.
% It also returns an exception flag equal to 1 if any share is negative.
% It also returns a vector, exclude, which has a single bit set to indicate
% the group to exclude when calculating the likelihood. Currently this bit
% is set for the splitter group.
% It also returns the identity of the actual group that is split,
% actualsplitter, (which may not be equal to the splitter) and the actual
% size of the group that is split. These are used to check the insider
% revolution constraint only.

% It takes as inputs:
%
% F - leadership payoff
% gamma - probability of successful coup
% rprime = delta*epsilon*(1-r)/r/(1-delta) where delta is the
%    discount factor, epsilon is the probability of regime change, and r is
%    the factor by which patronage is destroyed when a revolution occurs
% splitter - the ethnicity that is generally split (j)
% corecoal - ones indicate membership in the core coalition
% bigoutsiders - ones indicate the group > j is big enough to span the gap
% popshare - share of population (vector of size equal to number of ethnicities) 
% leader - current leader
% P - population size
% lambda - fraction of elites

% number of ethnicities
numeth  = size(popshare,1);

% get the elite sizes (a vector of the sizes of each ethnicity's elite)
elite = lambda*P.*popshare;

% get the sum of the shares of the population in the core coalition
coresum = corecoal'*elite;

% in the first step, we determine the splits required to calculate the
% payments in the standard case in which the leader is one of 1...splitter

% when the splitter is less than 2, the elite size for j-1 is zero
if splitter > 1
    sizejmin1 = elite(splitter-1);
else
    sizejmin1 = 0;
end;
    
% this is the split size for j in the standard case
stdsplitsize_j = 1 - rprime - coresum - sizejmin1;

% this is the split size for j-1 in the standard case (j is leader)
if splitter > 1
    stdsplitsize_jmin1 = 1 - rprime - coresum - elite(splitter);
else
    stdsplitsize_jmin1 = 0;
end;

% in the second step, calculate the shares according to the formula for
% 1...j-1 (will overwrite later for those that have different shares)

% gammaF is used in several places
gammaF = gamma*F/(1- gamma);

% ratio of split of j-1 over total for j-1
if splitter > 1
    jmin1ratio = stdsplitsize_jmin1/elite(splitter-1);
else
    % here, splitter is 1 so are dividing zero by zero, but inspecting the
    % formula for the share to j reveals that we should treat the ratio as
    % zero in this case
    jmin1ratio = 0;
end;

% payment to the splitter, j, in the standard coalition
stdshare_j = 1/(1-stdsplitsize_j/elite(splitter)*gamma^2*(splitter-2+jmin1ratio)/(1+gamma*(splitter-2)));
stdshare_j = stdshare_j*gamma*(stdsplitsize_j/elite(splitter)*(1-(splitter-2+jmin1ratio)*...
    gamma*(1-gammaF*(coresum + sizejmin1))/(1+gamma*(splitter-2))-...
    gammaF*(coresum+stdsplitsize_jmin1))+stdsplitsize_j*F);

% standard formula shares for 1...j-1
shares = gamma*(1-stdshare_j-gammaF*(coresum + sizejmin1))/(1+gamma*(splitter-2))+...
    gammaF.*elite;

% set shares for splitter to standard value to start with
shares(splitter) = stdshare_j;

% in the third step, on a case by case basis we need to change the shares
% when the leader is > j-1
% we also create a vector, coalvec, indicating membership in the coalition
% (to be used later to zero out shares for non-members); the leader is not
% included explicitly 
% finally, we create 'exclude' which has a one set for the group to be
% excluded in calculating the likelihood (the actual split group if there
% is one)

% coalition vector starts with core members
coalvec = corecoal;

% exclude is all zeros initially
exclude = zeros(numeth,1);

% proceed through cases

% start with standard case
if leader <= (splitter - 1)
    % note that this case can only occur when the splitter is > 1
    % include j-1
    coalvec(splitter-1) = 1;
    % include j
    coalvec(splitter) = 1;
    % all shares are correct already for this case
    exclude(splitter) = 1;
    actualsplitter = splitter;
    actualsplitsize = stdsplitsize_j;
% case in which the leader is j    
elseif leader == splitter
    if splitter > 1
        % include j-1
        coalvec(splitter-1) = 1;
        % scale payment of j-1 by amount of j-1 included (which is standard
        % split for j-1)
        shares(splitter-1) = shares(splitter - 1)*stdsplitsize_jmin1/elite(splitter - 1);
        exclude(splitter - 1) = 1;
        actualsplitter = splitter - 1;
        actualsplitsize = stdsplitsize_jmin1;
    else
        % nothing to be done here as leader=splitter=1 so it will be
        % handled later
        % the following values don't matter as here no insider constraint
        % is checked 
        actualsplitter = 0; %1
        actualsplitsize = 0;
    end;
    
% case in which the leader is > j and big enough to fill the gap
% note that bigoutsiders is only set for groups > j
elseif bigoutsiders(leader)==1
    if splitter > 1
        % include j-1
        coalvec(splitter-1) = 1;
        splitsize = 1 - rprime - coresum - elite(leader);
        % scale payment of j-1 by amount of j-1 included
        shares(splitter-1) = shares(splitter - 1)*splitsize/elite(splitter - 1);
        exclude(splitter - 1) = 1;
        actualsplitter = splitter - 1;
        actualsplitsize = splitsize;
    else
        % nothing to be done here as splitter=1 but leader itself is big
        % enough to lead on its own; handled later
        % the following values don't matter as here no insider constraint
        % is checked 
        actualsplitter = 0; %1
        actualsplitsize = 0;
    end;  
    
% case in which the leader is > j and not big enough to fill the gap     
else
    if splitter > 1
        % include j-1
        coalvec(splitter-1) = 1;
        % include j
        coalvec(splitter) = 1;
        splitsize = 1 - rprime - coresum - elite(splitter - 1) - elite(leader);
        % scale payment of j by amount of j included
        % scaling is relative to standard split size for j
        shares(splitter) = shares(splitter)*splitsize/stdsplitsize_j;
        actualsplitter = splitter;
        actualsplitsize = splitsize;
    else
        % splitter is 1 and needs to be split because leader is not big
        % enough to fill the gap on its own
        % include j
        coalvec(splitter) = 1;
        splitsize = 1 - rprime - coresum - elite(leader);
        % scale payment of j by amount of j included
        % scaling is relative to standard split size for j
        shares(splitter) = shares(splitter)*splitsize/stdsplitsize_j;
        actualsplitter = splitter;
        actualsplitsize = splitsize;
    end;    
    exclude(splitter) = 1;
end;

% in the fourth step, we overwrite the share for the leader and zero out
% the shares of non-coalition members

% zero out shares for non-members and for the leader so that we have only
% non-leader shares
coalvec(leader) = 0;
shares = shares.*coalvec;

% set leader shares to 1-sum of others' shares
shares(leader) = 1-ones(1,numeth)*shares;

% put leader back in coalition just for checking share values
coalvec(leader) = 1;

flag = 0;
% check that no shares are less than zero
if (sum(shares.*coalvec < 0))
    flag = 1;
end;

% check that no share is greater than 1
if (sum(shares.*coalvec > 1))
    flag = 1;
end;
    