function [ll] = windowlikelihood(param,popshare,govshare,regimes,includeobs,numeth,vectorize)

% This function calculates the (negative) log likelihood of the window
% dressing model where leaders allocate shares proportional to size but may
% keep a portion for themselves.
%
% Inputs:
%
% param: this is the parameter vector to be estimated [weight xi] where
% weight is the weight given to each group's size and xi is the parameter of the
% beta function error term
%
% popshare: vector of the population share for each ethnicity-time obs
%
% govshare: observed share of government positions for each ethnicity-time
% obs
%
% regimes: three column vector where first column is leader, second
% column is length of leadership, third column is observation in which
% transition occurred; one row per regime
%
% includeobs : observations to be included in the likelihood
%
% numeth: number of ethnicities (assumed fixed over time), used to delineate
% time periods
%
% vectorize : set to 1 to return individual log-likelihoods instead of sum

% extract parameters
weight = param(1);
xi = param(2);

% start by giving each a share equal to weight*size
predshare = popshare.*weight;
% now add in leadership premium for leaders (equal to 1-weight to ensure
% shares sum to 1)
for r=1:size(regimes,1)
    % loop over time periods within regime
    for t=1:regimes(r,2)
        % set leader's share to 1
        predshare((regimes(r,3)+(t-1)*numeth+regimes(r,1))-1,1) = predshare((regimes(r,3)+(t-1)*numeth+regimes(r,1))-1,1) + 1-weight;
    end;
end;

% identify those observed in the coalition
incoalition = (govshare>0);

% create vector of groups to exclude (smallest group)
nobs = size(popshare,1);
numperiods = nobs/numeth;
exclude = zeros(nobs,1); % initialization
for t=1:numperiods
    exclude(t*numeth)=1;
end

% calculate beta distribution probabilities for those in and out of the
% coalition
% because the support of the distribution we want, we need to scale the pdf
% and cdf as follows
% let y be the r.v. with support [-1,1] and we want to create x with
% support [0,1] such that x is distributed with a beta distribution
% we let x=(y+1)/2 and then the pdf of y is 1/2beta((y+1)/2) and the cdf of
% y is B((y+1)/2)
inpr = 1/2*betapdf((govshare-predshare+1)/2,xi,xi);
outpr = betacdf((1-predshare)/2,xi,xi);

% create a vector of each observation's contribution to the log-likelihood
% only consider non-excluded groups when adding to the likelihood
% don't let log go to negative infinity as it makes whole expression
% undefined even when that part of the expression is not used
llobs = includeobs.*(1-exclude).*(incoalition.*max(-1E5,log(inpr))+(1-incoalition).*max(-1E5,log(outpr)));

if vectorize == 1
    % return individual likelihoods
    ll = -llobs;
else
    % sum up likelihoods
    ll = -ones(1,size(popshare,1))*llobs;         
end;

